@@ -63,7 +63,7 @@
       (let* ((frq-scl (hz->radians freq))
 	     (maxdev (* frq-scl fm-index))
 	     (index1 (* maxdev (/ 5.0 (log freq))))
-	     (index2 (* maxdev 3.0 (/ (- 8.5 (log freq)) (+ 3.0 (/ freq 1000)))))
+	     (index2 (/ (* maxdev 3.0 (- 8.5 (log freq))) (+ 3.0 (/ freq 1000))))
 	     (index3 (* maxdev (/ 4.0 (sqrt freq))))
 	     (carrier (make-oscil :frequency freq))
 	     (fmosc1 (make-oscil :frequency freq))
@@ -101,8 +101,8 @@
 	(dur 0.0))
     (do ((i 0 (+ 1 i)))
 	((= i lim))
-      (set! (vpits i) (floor (random 12.0)))
-      (set! (vbegs i) (+ 1 (floor (random 3.0)))))
+      (set! (vpits i) (random 12))
+      (set! (vbegs i) (+ 1 (random 3))))
     (set! *clm-srate* srate)
     (set! *clm-rt-bufsize* bufsize)
     (set! *output* (mus-audio-open-output mus-audio-default srate 1 sample-type (* bufsize 2)))
@@ -118,7 +118,7 @@
 		  (if (> (random 1.0) 0.5) (set! cellsiz (+ 1 cellsiz)))
 		  (set! cellctr cellbeg)))
 
-	    (format #t "dur: ~A, freq: ~A, amp: ~A, index: ~A~%"
+	    (format () "dur: ~A, freq: ~A, amp: ~A, index: ~A~%"
 		    dur
 		    (let ((freq (* cfreq 16.351 16
 				   (expt 2 (/ (vmode (vpits cellctr))
@@ -153,10 +153,8 @@
 (define amps (make-vector (+ 1 lim) 0))
 
 (define (tune x)
-  (let* ((pit (modulo x 12))
-	 (oct (floor (/ x 12)))
-	 (base (rats pit)))
-    (* base (expt 2 oct))))
+  (* (rats (modulo x 12))
+     (expt 2 (floor (/ x 12)))))
 
 (define (rbell x)
   (envelope-interp (* x 100) bell))
@@ -189,11 +187,11 @@
     (do ((i 0 (+ i 1)))
 	((= i lim))
       (set! (octs i) (floor (+ 4 (* 2 (rbell (random 1.0))))))
-      (set! (pits i) (mode (floor (random 12.0))))
-      (set! (rhys i) (floor (+ 4 (random 6.0))))
+      (set! (pits i) (mode (random 12)))
+      (set! (rhys i) (+ 4 (random 6)))
       (set! (begs i) (if (< (random 1.0) 0.9) 
-		      (floor (+ 4 (random 2.0)))
-		      (floor (random 24.0))))
+			 (+ 4 (random 2))
+			 (random 24)))
       (set! (amps i) (floor (+ 1 (* 8 (rbell (random 1.0)))))))
     (set! *clm-srate* srate)
     (set! *clm-rt-bufsize* bufsize)
@@ -213,12 +211,14 @@
 		(begin
 		  (set! cellbeg (+ 1 cellbeg))
 		  (if (> (random 1.0) 0.5) (set! cellsiz (+ cellsiz whichway)))
-		  (if (and (> cellsiz 10) (> (random 1.0) 0.99))
-		      (set! whichway -2)
-		      (if (and (> cellsiz 6) (> (random 1.0) 0.999))
-			  (set! whichway -1)
-			  (if (< cellsiz 4)
-			      (set! whichway 1))))
+		  (cond ((and (> cellsiz 10)
+			      (> (random 1.0) 0.99))
+			 (set! whichway -2))
+			((and (> cellsiz 6)
+			      (> (random 1.0) 0.999))
+			 (set! whichway -1))
+			((< cellsiz 4) 
+			 (set! whichway 1)))
 		  (set! nextbeg (+ nextbeg (random 1.0)))
 		  (set! cellctr cellbeg)))
 	    (set! func (make-rt-violin dur freq ampl
@@ -263,11 +263,11 @@
 	 (shell (car shell-app))
 	 (dpy (XtDisplay shell))
 	 (screen (DefaultScreenOfDisplay dpy))
-	 (cmap (DefaultColormap dpy (DefaultScreen dpy)))
 	 (black (BlackPixelOfScreen screen)))
 
     (define (get-color color)
-      (let ((col (XColor)))
+      (let ((col (XColor))
+	    (cmap (DefaultColormap dpy (DefaultScreen dpy))))
 	(if (= (XAllocNamedColor dpy cmap color col col) 0)
 	    (error (format #f "can't allocate ~A" color))
 	    (.pixel col))))
@@ -444,7 +444,6 @@
 						     XmNbackground       light-blue)))
 	   (low-tempo 0.05)
 	   (high-tempo 0.5)
-	   (high-amp 1.0)
 	   (low-freq 0.1)
 	   (high-freq 4.0)
 	   (high-index 2.0)
@@ -457,7 +456,8 @@
 	(set-flabel tempo-label ctempo))
 
       (define (amp-callback w c i)
-	(set! camp (* (.value i) (/ high-amp 100.0)))
+	(let ((high-amp 1.0))
+	  (set! camp (* (.value i) (/ high-amp 100.0))))
 	(set-flabel amp-label camp))
 
       (define (freq-callback w c i)
@@ -523,9 +523,7 @@
 		       (if cplay
 			   (begin
 			     (set-defaults)
-			     (if (= which-play 0)
-				 (set! func (apply make-agn (or args ())))
-				 (set! func (apply make-float-vector-test (or args ()))))
+			     (set! func (apply (if (= which-play 0) make-agn make-float-vector-test) (or args ())))
 			     (set! proc (XtAppAddWorkProc app (lambda (c) (rt-send->dac func)))))
 			   (if proc (XtRemoveWorkProc proc)))))
       (XmToggleButtonSetState play-button cplay #f)
