@@ -72,20 +72,20 @@
 	       (x0 0)
 	       (y0 0)
 	       (line-ctr 2)
-	       (lines (make-vector (* 2 (+ 1 (- (axinf 12) (axinf 10)))) 0)))
+	       (lines (make-vector (* 2 (- (+ (axinf 12) 1) (axinf 10))) 0)))
 	  (dynamic-wind
 	      (lambda ()
 		(set! (foreground-color snd chn) red))
 	      (lambda ()
-		(if (< start left)                ; check previous samples to get first rms value
-		    (do ((i start (+ 1 i)))
+		(if (< start left)                 ; check previous samples to get first rms value
+		    (do ((i start (+ 1 i))) 
 			((= i left))
 		      (moving-rms rms (reader))))
 		(let ((first-sample (next-sample reader)))
 		  (set! x0 (grf-it (* left sr) xdata))
-		  (set! y0 (grf-it first-sample ydata))
-		  (set! (lines 0) x0)        ; first graph point
-		  (set! (lines 1) y0))
+		  (set! y0 (grf-it first-sample ydata)))
+		(set! (lines 0) x0)                ; first graph point
+		(set! (lines 1) y0)
 		(do ((i (+ left 1) (+ 1 i)))       ; loop through all samples calling moving-rms
 		    ((= i right))
 		  (let ((x1 (grf-it (* i sr) xdata))
@@ -121,29 +121,29 @@ whenever they're in the current view."))
 	    (end (+ beg dur))
 	    (old-color (foreground-color snd chn))
 	    (cr (make-cairo (car (channel-widgets snd chn)))))
-	(if (and (< left end)
-		 (> right beg))
-	    (let ((data (make-graph-data snd chn)))
-	      (if (float-vector? data)
-		  (let* ((samps (- (min right end) (max left beg)))
-			 (offset (max 0 (- beg left)))
-			 (new-data (float-vector-subseq data offset (+ offset samps))))
-		    (set! (foreground-color snd chn) color)
-		    (graph-data new-data snd chn copy-context (max beg left) (min end right) (time-graph-style snd chn) cr)
-		    (set! (foreground-color snd chn) old-color))
-		  (let* ((low-data (car data))
-			 (high-data (cadr data))
-			 (size (length low-data))
-			 (samps (- right left))
-			 (left-offset (max 0 (- beg left)))
-			 (left-bin (floor (/ (* size left-offset) samps)))
-			 (right-offset (- (min end right) left))
-			 (right-bin (floor (/ (* size right-offset) samps)))
-			 (new-low-data (float-vector-subseq low-data left-bin right-bin))
-			 (new-high-data (float-vector-subseq high-data left-bin right-bin)))
-		    (set! (foreground-color snd chn) color)
-		    (graph-data (list new-low-data new-high-data) snd chn copy-context left-bin right-bin (time-graph-style snd chn) cr)
-		    (set! (foreground-color snd chn) old-color)))))
+	(when (and (< left end)
+		   (> right beg))
+	  (let ((data (make-graph-data snd chn)))
+	    (if (float-vector? data)
+		(let* ((samps (- (min right end) (max left beg)))
+		       (offset (max 0 (- beg left)))
+		       (new-data (float-vector-subseq data offset (+ offset samps))))
+		  (set! (foreground-color snd chn) color)
+		  (graph-data new-data snd chn copy-context (max beg left) (min end right) (time-graph-style snd chn) cr)
+		  (set! (foreground-color snd chn) old-color))
+		(let* ((low-data (car data))
+		       (high-data (cadr data))
+		       (size (length low-data))
+		       (samps (- right left))
+		       (left-offset (max 0 (- beg left)))
+		       (left-bin (floor (/ (* size left-offset) samps)))
+		       (right-offset (- (min end right) left))
+		       (right-bin (floor (/ (* size right-offset) samps)))
+		       (new-low-data (float-vector-subseq low-data left-bin right-bin))
+		       (new-high-data (float-vector-subseq high-data left-bin right-bin)))
+		  (set! (foreground-color snd chn) color)
+		  (graph-data (list new-low-data new-high-data) snd chn copy-context left-bin right-bin (time-graph-style snd chn) cr)
+		  (set! (foreground-color snd chn) old-color)))))
 	(free-cairo cr)))))
 
 
@@ -153,7 +153,7 @@ whenever they're in the current view."))
     ;; intended as after-graph-hook member 
     ;; run through 'colored-samples lists passing each to display-colored-samples
     (let ((colors (channel-property 'colored-samples snd chn)))
-      (if colors
+      (if (pair? colors)
 	  (for-each
 	   (lambda (vals)
 	     (apply display-colored-samples (append vals (list snd chn))))
@@ -177,37 +177,36 @@ whenever they're in the current view."))
 (define uncolor-samples 
   (let ((documentation "(uncolor-samples snd chn) cancels sample coloring in the given channel"))
     (lambda* (usnd uchn)
-      (let*	((snd (or usnd (selected-sound) (car (sounds))))
-		 (chn (or uchn (selected-channel snd) 0)))
+      (let* ((snd (or usnd (selected-sound) (car (sounds))))
+	     (chn (or uchn (selected-channel snd) 0)))
 	(set! (channel-property 'colored-samples snd chn) ())
 	(update-time-graph snd chn)))))
 
 
-
 (define display-previous-edits 
   (let ((documentation "(display-previous-edits snd chn) displays all edits of the current sound, with older versions gradually fading away"))
     (lambda (snd chn)
       (let ((edits (edit-position snd chn)))
-	(if (> edits 0)
-	    (let* ((old-color (foreground-color snd chn))
-		   (clist (color->list old-color))
-		   (r (car clist))
-		   (g (cadr clist))
-		   (b (caddr clist))
-		   (rinc (/ (- 1.0 r) (+ edits 1)))
-		   (ginc (/ (- 1.0 g) (+ edits 1)))
-		   (binc (/ (- 1.0 b) (+ edits 1)))
-		   (cr (make-cairo (car (channel-widgets snd chn))))) 
-	      (do ((pos 0 (+ 1 pos))
-		   (re (- 1.0 rinc) (- re rinc))
-		   (ge (- 1.0 ginc) (- ge ginc))
-		   (be (- 1.0 binc) (- be binc)))
-		  ((> pos edits))
-		(let ((data (make-graph-data snd chn pos)))
-		  (set! (foreground-color snd chn) (make-color re ge be))
-		  (graph-data data snd chn copy-context #f #f (time-graph-style snd chn) cr)))
-	      (set! (foreground-color snd chn) old-color)
-	      (free-cairo cr)))))))
+	(when (> edits 0)
+	  (let* ((old-color (foreground-color snd chn))
+		 (clist (color->list old-color))
+		 (r (car clist))
+		 (g (cadr clist))
+		 (b (caddr clist))
+		 (rinc (/ (- 1.0 r) (+ edits 1)))
+		 (ginc (/ (- 1.0 g) (+ edits 1)))
+		 (binc (/ (- 1.0 b) (+ edits 1)))
+		 (cr (make-cairo (car (channel-widgets snd chn))))) 
+	    (do ((pos 0 (+ 1 pos))
+		 (re (- 1.0 rinc) (- re rinc))
+		 (ge (- 1.0 ginc) (- ge ginc))
+		 (be (- 1.0 binc) (- be binc)))
+		((> pos edits))
+	      (let ((data (make-graph-data snd chn pos)))
+		(set! (foreground-color snd chn) (make-color re ge be))
+		(graph-data data snd chn copy-context #f #f (time-graph-style snd chn) cr)))
+	    (set! (foreground-color snd chn) old-color)
+	    (free-cairo cr)))))))
 
 
 (define overlay-sounds
@@ -234,35 +233,35 @@ whenever they're in the current view."))
 (define samples-via-colormap 
   (let ((documentation "(samples-via-colormap snd chn) displays time domain graph using current colormap (just an example of colormap-ref)"))
     (lambda (snd chn)
-      (let ((left (left-sample snd chn))
-	    (right (right-sample snd chn))
-	    (old-color (foreground-color snd chn))
-	    (data (make-graph-data snd chn))
+      (let ((data (make-graph-data snd chn))
 	    (cr (make-cairo (car (channel-widgets snd chn)))))
 	
 	(define (samples-1 cur-data)
-	  (let* ((x0 (x->position (/ left (srate snd))))
-		 (y0 (y->position (cur-data 0)))
-		 (colors (make-vector *colormap-size* #f))
-		 (len (length cur-data))
-		 (incr (/ (+ 1 (- right left)) len)))
-	    (do ((i (+ left incr) (+ i incr))
-		 (j 1 (+ 1 j)))
-		((or (>= i right)
-		     (>= j len)))
-	      (let* ((x1 (x->position (/ i (srate snd))))
-		     (y1 (y->position (cur-data j)))
-		     (x (abs (cur-data j)))
-		     (ref (floor (* *colormap-size* x)))
-		     (color (or (colors ref)
-				(let ((new-color (apply make-color (colormap-ref (colormap) x))))
-				  (set! (colors ref) new-color)))))
-		(set! (foreground-color snd chn) color)
-		(draw-line x0 y0 x1 y1 snd chn time-graph cr)
-		(set! x0 x1)
-		(set! y0 y1)))
-	    (set! (foreground-color snd chn) old-color)))
-	
+	  (let ((left (left-sample snd chn))
+		(right (right-sample snd chn))
+		(old-color (foreground-color snd chn))
+		(y0 (y->position (cur-data 0)))
+		(colors (make-vector *colormap-size* #f))
+		(len (length cur-data)))
+	    (let ((x0 (x->position (/ left (srate snd))))
+		  (incr (/ (- (+ right 1) left) len)))
+	      (do ((i (+ left incr) (+ i incr))
+		   (j 1 (+ 1 j)))
+		  ((or (>= i right)
+		       (>= j len)))
+		(let ((x1 (x->position (/ i (srate snd))))
+		      (y1 (y->position (cur-data j))))
+		  (let* ((x (abs (cur-data j)))
+			 (ref (floor (* *colormap-size* x))))
+		    (set! (foreground-color snd chn) 
+			  (or (colors ref)
+			      (let ((new-color (apply make-color (colormap-ref (colormap) x))))
+				(set! (colors ref) new-color)))))
+		  (draw-line x0 y0 x1 y1 snd chn time-graph cr)
+		  (set! x0 x1)
+		  (set! y0 y1)))
+	      (set! (foreground-color snd chn) old-color))))
+
 	(if data
 	    (if (float-vector? data)
 		(samples-1 data)
