@@ -71,7 +71,7 @@
     (lambda ()
       (let ((snc (sync)))
 	(define (trim-front-one-channel snd chn)
-	  (if (< (length (marks snd chn)) 1)
+	  (if (null? (marks snd chn))
 	      (status-report "trim-front needs a mark" snd)
 	      (delete-samples 0 (mark-sample (car (marks snd chn))) snd chn)))
 	(if (> snc 0)
@@ -91,7 +91,7 @@
     (lambda ()
       (let ((snc (sync)))
 	(define (trim-back-one-channel snd chn)
-	  (if (< (length (marks snd chn)) 1)
+	  (if (null? (marks snd chn))
 	      (status-report "trim-back needs a mark" snd)
 	      (let ((endpt (mark-sample (car (reverse (marks snd chn))))))
 		(delete-samples (+ endpt 1) (- (framples snd chn) endpt)))))
@@ -139,35 +139,31 @@
 
 ;;; -------- add these to the Edit menu, if possible
 
-(if (and (not (provided? 'snd-gtk))
-	 (provided? 'xm))
-    (with-let (sublet *motif*)
-      (let* ((edit-cascade (list-ref (menu-widgets) 2))
-	     (edit-menu (cadr (XtGetValues edit-cascade (list XmNsubMenuId 0)))))
-	
-	(define (for-each-child w func)
-	  ;; (for-each-child w func) applies func to w and its descendents
-	  (func w)
-	  (if (XtIsComposite w)
-	      (for-each 
-	       (lambda (n)
-		 (for-each-child n func))
-	       (cadr (XtGetValues w (list XmNchildren 0) 1)))))
-	
-	(XtAddCallback edit-cascade XmNcascadingCallback 
-		       (lambda (w c i)
-			 (for-each-child 
-			  edit-menu
-			  (lambda (child)
-			    (if (or (string=? (XtName child) "Selection->new")
-				    (string=? (XtName child) "Cut selection->new")
-				    (string=? (XtName child) "Append selection"))
-				(XtSetSensitive child (selection?))
-				(if (string=? (XtName child) "Crop")
-				    (XtSetSensitive child (and (selected-sound)
-							       (> (length (marks (selected-sound) (selected-channel))) 1)))
-				    (if (or (string=? (XtName child) "Trim front")
-					    (string=? (XtName child) "Trim back"))
-					(XtSetSensitive child (and (selected-sound)
-								   (>= (length (marks (selected-sound) (selected-channel))) 1))))))))))))
-    )
+(when (and (not (provided? 'snd-gtk))
+	   (provided? 'xm))
+  (with-let (sublet *motif*)
+    (let* ((edit-cascade (list-ref (menu-widgets) 2))
+	   (edit-menu (cadr (XtGetValues edit-cascade (list XmNsubMenuId 0)))))
+      
+      (define (for-each-child w func)
+	;; (for-each-child w func) applies func to w and its descendents
+	(func w)
+	(if (XtIsComposite w)
+	    (for-each 
+	     (lambda (n)
+	       (for-each-child n func))
+	     (cadr (XtGetValues w (list XmNchildren 0) 1)))))
+      
+      (XtAddCallback edit-cascade XmNcascadingCallback 
+		     (lambda (w c i)
+		       (for-each-child 
+			edit-menu
+			(lambda (child)
+			  (cond ((member (XtName child) '("Selection->new" "Cut selection->new" "Append selection") string=?)
+				 (XtSetSensitive child (selection?)))
+				((string=? (XtName child) "Crop")
+				 (XtSetSensitive child (and (selected-sound)
+							    (> (length (marks (selected-sound) (selected-channel))) 1))))
+				((member (XtName child) '("Trim front" "Trim back") string=?)
+				 (XtSetSensitive child (and (selected-sound)
+							    (>= (length (marks (selected-sound) (selected-channel))) 1))))))))))))
