@@ -167,8 +167,7 @@
 	      (srate file)
 	      (mus-header-type-name (mus-sound-header-type file))
 	      (mus-sample-type-name (mus-sound-sample-type file))
-	      (/ (mus-sound-samples file)
-		 (* 1.0 (channels file) (srate file)))))))
+	      (* 1.0 (/ (mus-sound-samples file) (channels file) (srate file)))))))
 
 
 ;;; -------- Correlation --------
@@ -179,9 +178,10 @@
   (let ((documentation "(display-correlation hook) returns the correlation of snd's 2 channels (intended for use with graph-hook).  y0 and y1 are ignored."))
     (lambda (hook)
       (let ((snd (hook 'snd)))
-	(if (and (= (channels snd) 2)
-		 (> (framples snd 0) 1)
-		 (> (framples snd 1) 1))
+	(if (not (and (= (channels snd) 2)
+		      (> (framples snd 0) 1)
+		      (> (framples snd 1) 1)))
+	    (status-report "display-correlation wants stereo input")
 	    (let* ((ls (left-sample snd 0))
 		   (rs (right-sample snd 0))
 		   (ilen (+ 1 (- rs ls)))
@@ -204,8 +204,7 @@
 		(float-vector-subtract! im2 rl2)       ; subtract the 4th from the 3rd
 		(fft tmprl im2 -1)
 		(float-vector-scale! tmprl fftscale)   ; scale by fftscale
-		(graph tmprl "lag time" 0 fftlen)))
-	    (status-report "display-correlation wants stereo input"))))))
+		(graph tmprl "lag time" 0 fftlen))))))))
 
 					;(hook-push graph-hook display-correlation)
 
@@ -282,8 +281,8 @@
 	     (b2 (char->integer (read-char fd)))
 	     (b3 (char->integer (read-char fd))))
 	(close-input-port fd)
-	(if (or (not (= b0 255))
-		(not (= (logand b1 #b11100000) #b11100000)))
+	(if (not (and (= b0 255)
+		      (= (logand b1 #b11100000) #b11100000)))
 	    (snd-print (format #f "~S is not an MPEG file (first 11 bytes: #b~B #b~B)" mpgfile b0 (logand b1 #b11100000)))
 	    (let ((id (ash (logand b1 #b11000) -3))
 		  (layer (ash (logand b1 #b110) -1))
@@ -304,7 +303,7 @@
 	      (let* ((chans (if (= channel-mode 3) 1 2))
 		     (mpegnum (if (= id 0) 4 (if (= id 2) 2 1)))
 		     (mpeg-layer (if (= layer 3) 1 (if (= layer 2) 2 3)))
-		     (srate (/ (list-ref (list 44100 48000 32000 0) srate-index) mpegnum)))
+		     (srate (/ (#(44100 48000 32000 0) srate-index) mpegnum)))
 		(snd-print (format #f "~S: ~A Hz, ~A, MPEG-~A~%" 
 				   mpgfile srate (if (= chans 1) "mono" "stereo") mpeg-layer))
 		(system (format #f "mpg123 -s ~A > ~A" mpgfile rawfile))
@@ -322,10 +321,7 @@
       ;; (open-sound (read-ogg "/home/bil/sf1/oboe.ogg"))
       (and (call-with-input-file filename 
 	     (lambda (fd)
-	       (and (char=? (read-char fd) #\O)
-		    (char=? (read-char fd) #\g)
-		    (char=? (read-char fd) #\g)
-		    (char=? (read-char fd) #\S))))
+	       (string=? (read-string 4 fd) "OggS")))
 	   (let ((aufile (string-append filename ".au")))
 	     (if (file-exists? aufile) (delete-file aufile))
 	     (system (format #f "ogg123 -d au -f ~A ~A" aufile filename))
@@ -373,7 +369,7 @@
 	    (save-sound-as file snd :header-type mus-riff)
 	    (system (format #f "speexenc ~A ~A" file spxfile))
 	    (delete-file file))
-	  (system (format #f "speexenc ~A ~A" (file-name snd) (string-append (file-name snd) ".spx")))))))
+	  (system (format #f "speexenc ~A ~A.spx" (file-name snd) (file-name snd)))))))
 
 
 ;;; -------- read and write FLAC files
@@ -449,15 +445,11 @@ read an ASCII sound file"))
 	     (chn (hook 'chn))
 	     (dots (- (right-sample snd chn)
 		      (left-sample snd chn))))
-	(if (> dots 100) 
-	    (set! (dot-size snd chn) 1)
-	    (if (> dots 50)
-		(set! (dot-size snd chn) 2)
-		(if (> dots 25)
-		    (set! (dot-size snd chn) 3)
-		    (set! (dot-size snd chn) 5))))))))
+	(set! (dot-size snd chn)
+	      (cond ((assoc dots '((100 . 1) (50 . 2) (25 . 3)) >) => cdr) 
+		    (else 5)))))))
 
-					;(hook-push graph-hook auto-dot)
+;;; (hook-push graph-hook auto-dot)
 
 
 
@@ -473,14 +465,15 @@ read an ASCII sound file"))
     (lambda ()
       (let* ((keysnd (or (selected-sound) (car (sounds))))
 	     (keychn (or (selected-channel keysnd) 0))
-	     (current-left-sample (left-sample keysnd keychn))
 	     (chan-marks (marks keysnd keychn)))
-	(define (find-leftmost-mark samps)
-	  (and (pair? samps)
-	       (if (> (car samps) current-left-sample)
-		   (car samps)
-		   (find-leftmost-mark (cdr samps)))))
-	(if (= (length chan-marks) 0)
+	(define find-leftmost-mark 
+	  (let ((current-left-sample (left-sample keysnd keychn)))
+	    (lambda (samps)
+	      (and (pair? samps)
+		   (if (> (car samps) current-left-sample)
+		       (car samps)
+		       (find-leftmost-mark (cdr samps)))))))
+	(if (null? chan-marks)
 	    (status-report "no marks!")
 	    (let ((leftmost (find-leftmost-mark (map mark-sample chan-marks))))
 	      (if (number? leftmost)
@@ -488,7 +481,7 @@ read an ASCII sound file"))
 		    (set! (left-sample keysnd keychn) leftmost)
 		    keyboard-no-action)
 		  (status-report "no mark in window"))))))))
-
+    
 					;(bind-key #\m 0 (lambda () "align window left edge with mark" (first-mark-in-window-at-left)))
 
 
@@ -537,11 +530,10 @@ two sounds open (indices 0 and 1 for example), and the second has two channels,
       (let ((sndlist ())
 	    (chnlist ()))
 	(for-each (lambda (snd)
-		    (let ((chntop (- (channels snd) 1)))
-		      (do ((i chntop (- i 1)))
-			  ((< i 0))
-			(set! sndlist (cons snd sndlist))
-			(set! chnlist (cons i chnlist)))))
+		    (do ((i (- (channels snd) 1) (- i 1)))
+			((< i 0))
+		      (set! sndlist (cons snd sndlist))
+		      (set! chnlist (cons i chnlist))))
 		  (sounds))
 	(list sndlist chnlist)))))
 
@@ -620,11 +612,11 @@ otherwise it moves the cursor to the first offending sample"))
 it into two copies whose amplitudes depend on the envelope 'pan-env'.  If 'pan-env' is 
 a number, the sound is split such that 0 is all in channel 0 and 90 is all in channel 1."))
     (lambda (mono-snd stereo-snd pan-env)
-      (let ((len (framples mono-snd)))
+      (let ((len (framples mono-snd))
+	    (reader0 (make-sampler 0 mono-snd))
+	    (reader1 (make-sampler 0 mono-snd)))
 	(if (number? pan-env)
-	    (let ((pos (/ pan-env 90.0))
-		  (reader0 (make-sampler 0 mono-snd))
-		  (reader1 (make-sampler 0 mono-snd)))
+	    (let ((pos (/ pan-env 90.0)))
 	      (map-channel (lambda (y)
 			     (+ y (* pos (read-sample reader1))))
 			   0 len stereo-snd 1)
@@ -632,9 +624,7 @@ a number, the sound is split such that 0 is all in channel 0 and 90 is all in ch
 			     (+ y (* (- 1.0 pos) (read-sample reader0))))
 			   0 len stereo-snd 0))
 	    (let ((e0 (make-env pan-env :length len))
-		  (e1 (make-env pan-env :length len))
-		  (reader0 (make-sampler 0 mono-snd))
-		  (reader1 (make-sampler 0 mono-snd)))
+		  (e1 (make-env pan-env :length len)))
 	      (map-channel (lambda (y)
 			     (+ y (* (env e1) (read-sample reader1))))
 			   0 len stereo-snd 1)
@@ -657,8 +647,8 @@ then inverse ffts."))
 	     (fsize2 (/ fsize 2))
 	     (rdata (channel->float-vector 0 fsize snd chn))
 	     (idata (make-float-vector fsize))
-	     (lo (round (/ bottom (/ sr fsize))))
-	     (hi (round (/ top (/ sr fsize)))))
+	     (lo (round (/ (* bottom fsize) sr)))
+	     (hi (round (/ (* top fsize) sr))))
 	(fft rdata idata 1)
 	(if (> lo 0)
 	    (begin
@@ -779,12 +769,11 @@ then inverse ffts."))
 				 ((= i fft-size))
 			       (float-vector-set! rl i (read-sample read-ahead)))
 			     (fill! im 0.0)))
-		       (let ((rval (- 1.0 (ramp ramper in-vowel))))
+		       (* y (- 1.0 (ramp ramper in-vowel))))
 					; squelch consonants if just ramp value (not 1.0-val)
 					;(and (> rval 0.0) ; if this is included, the vowel-portions are omitted
-			 (* y rval) ; squelch vowels 
+					; squelch vowels 
 					;(* y (+ (* 2 rval) .1)) ;accentuate consonants
-			 ))
 		     0 #f snd chn #f "squelch-vowels")))))
 
 
@@ -855,10 +844,11 @@ current spectrum value.  (filter-fft (lambda (y) (if (< y .01) 0.0 y))) is like
 	(float-vector-multiply! rdata vf)
 	(float-vector-multiply! idata vf)
 	(fft rdata idata -1)
-	(if (not (= mx 0.0))
+	(if (= mx 0.0)
+	    (float-vector->channel rdata 0 (- len 1) snd chn #f (format #f "filter-fft ~A" flt))
 	    (let ((pk (float-vector-peak rdata)))
-	      (float-vector->channel (float-vector-scale! rdata (/ mx pk)) 0 (- len 1) snd chn #f (format #f "filter-fft ~A" flt)))
-	    (float-vector->channel rdata 0 (- len 1) snd chn #f (format #f "filter-fft ~A" flt)))))))
+	      (float-vector->channel (float-vector-scale! rdata (/ mx pk)) 0 (- len 1) snd chn #f (format #f "filter-fft ~A" flt))))))))
+	    
 
 ;; (let ((op (make-one-zero .5 .5))) (filter-fft op))
 ;; (let ((op (make-one-pole .05 .95))) (filter-fft op))
@@ -1290,8 +1280,7 @@ selected sound: (map-channel (cross-synthesis (integer->sound 0) .5 128 6.0))"))
 	  (if (odd? ctr) (set! ctr (- ctr 1)))
 	  
 	  (set! fdr (channel->float-vector inctr fftsize snd chn))
-	  (let ((pk (float-vector-peak fdr)))
-	    (if (> pk old-peak-amp) (set! old-peak-amp pk)))
+	  (set! old-peak-amp (max (float-vector-peak fdr) old-peak-amp))
 	  (spectrum fdr fdi #f 2)
 	  (float-vector-subtract! fdr spectr)
 	  (float-vector-scale! fdr (/ 2.0 freq-inc))
@@ -1337,8 +1326,7 @@ selected sound: (map-channel (cross-synthesis (integer->sound 0) .5 128 6.0))"))
 	  (set! ctr (min (- len i) freq-inc))
 	  
 	  (set! fdr (channel->float-vector inctr fftsize snd chn))
-	  (let ((pk (float-vector-peak fdr)))
-	    (if (> pk old-peak-amp) (set! old-peak-amp pk)))
+	  (set! old-peak-amp (max (float-vector-peak fdr) old-peak-amp))
 	  (spectrum fdr fdi #f 2)
 	  (float-vector-subtract! fdr spectr)
 	  (float-vector-scale! fdr (/ 1.0 freq-inc))
@@ -1389,12 +1377,12 @@ selected sound: (map-channel (cross-synthesis (integer->sound 0) .5 128 6.0))"))
       (let* ((start (cursor))
 	     (sf (make-sampler start)))
 	(do ((n start (+ 1 n))
-	     (val0 (abs (next-sample sf)) val1)
+	     (val0 (abs (next-sample sf)))
 	     (val1 (abs (next-sample sf)) (abs (next-sample sf))))
 	    ((or (sampler-at-end? sf)
 		 (< (+ val0 val1) limit))
-	     (set! (cursor) n)
-	     n))))))
+	     (set! (cursor) n))
+	  (set! val0 val1))))))
 
 
 ;;; -------- sound interp
@@ -1455,16 +1443,16 @@ selected sound: (map-channel (cross-synthesis (integer->sound 0) .5 128 6.0))"))
       ;; since the old/new sounds can be any length, we'll write a temp file rather than trying to use map-channel
       
       (let* ((len (framples snd chn))
-	     (newlen (floor (* time-scale len))))
-	(let ((new-snd (with-sound ((snd-tempnam) :to-snd #f :srate (srate snd))
+	     (newlen (floor (* time-scale len)))
+	     (new-snd (with-sound ((snd-tempnam) :to-snd #f :srate (srate snd))
 			 (let ((data (channel->float-vector 0 #f snd chn))
 			       (read-env (make-env envelope :length (+ 1 newlen) :scaler len)))
 			   (do ((i 0 (+ i 1)))
 			       ((= i newlen))
 			     (outa i (array-interp data (env read-env) len)))))))
-	  (set-samples 0 newlen new-snd snd chn #t
-		       (format #f "env-sound-interp '~A ~A" envelope time-scale)
-		       0 current-edit-position #t))))))
+	(set-samples 0 newlen new-snd snd chn #t
+		     (format #f "env-sound-interp '~A ~A" envelope time-scale)
+		     0 current-edit-position #t)))))
 
 
 ;;; (env-sound-interp '(0 0 1 1 2 0) 2.0)
@@ -1507,10 +1495,10 @@ the given channel following 'envelope' (as in env-sound-interp), using grains to
 				 (set! (mus-location read-env) i)
 				 (let ((position-in-original (env read-env)))
 				   (set! (readers next-reader)
-					 (make-sampler (max 0 (round (+ position-in-original (mus-random jitter)))) snd chn))
-				   (mus-reset (grain-envs next-reader)) ; restart grain env
-				   (set! next-reader (modulo (+ next-reader 1) num-readers))
-				   (if (< cur-readers next-reader) (set! cur-readers next-reader)))
+					 (make-sampler (max 0 (round (+ position-in-original (mus-random jitter)))) snd chn)))
+				 (mus-reset (grain-envs next-reader)) ; restart grain env
+				 (set! next-reader (modulo (+ next-reader 1) num-readers))
+				 (set! cur-readers (max cur-readers next-reader))
 				 
 				 (do ((k 0 (+ k 1)))
 				     ((= k cur-readers))
@@ -1607,13 +1595,12 @@ as env moves to 0.0, low-pass gets more intense; amplitude and low-pass amount m
   (let ((documentation "(remove-clicks) tries to find and smooth-over clicks"))
     (lambda ()
       ;; this is very conservative -- the click detection limits above could be set much tighter in many cases
-      (define (remove-click loc)
-	(let ((click (find-click loc)))
-	  (if click
-	      (begin
-		(smooth-sound (- click 2) 4)
-		(remove-click (+ click 2))))))
-      (remove-click 0))))
+      (let remove-click ((loc 0))
+        (let ((click (find-click loc)))
+          (if click
+              (begin
+                (smooth-sound (- click 2) 4)
+                (remove-click (+ click 2)))))))))
 
 
 ;;; -------- searching examples (zero+, next-peak)
@@ -1700,7 +1687,7 @@ In most cases, this will be slightly offset from the true beginning of the note"
 				     (begin
 				       (set! pk (spectr i))
 				       (set! pkloc i))))))
-			  (if (< (abs (- pitch pit)) (/ (srate) (* 2 *transform-size*))) ; uh... why not do it direct?
+			  (if (< (abs (- pitch pit)) (/ (srate) 2 *transform-size*)) ; uh... why not do it direct?
 			      (set! rtn #t)))))
 		  (fill! data 0.0)))
 	    rtn))))))
@@ -1763,30 +1750,27 @@ a sort of play list: (region-play-list (list (list reg0 0.0) (list reg1 0.5) (li
 (define explode-sf2
   (let ((documentation "(explode-sf2) turns the currently selected soundfont file into a bunch of files of the form sample-name.aif"))
     (lambda ()
-      (letrec ((sf2it 
-		(lambda (lst)
-		  (if (pair? lst)
-		      (let* ((vals (car lst))
-			     ;; each inner list is: '(name start loop-start loop-end)
-			     (name (car vals))
-			     (start (cadr vals))
-			     (end (if (null? (cdr lst))
-				      (framples)
-				      (cadadr lst)))
-			     (loop-start (- (caddr vals) start))
-			     (loop-end (- (cadddr vals) start))
-			     (filename (string-append name ".aif")))
-			(if (selection?)
-			    (set! (selection-member? #t) #f)) ; clear entire current selection, if any
-			(set! (selection-member?) #t)
-			(set! (selection-position) start)
-			(set! (selection-framples) (- end start))
-			(save-selection filename (selection-srate) mus-bshort mus-aifc)
-			(let ((temp (open-sound filename)))
-			  (set! (sound-loop-info temp) (list loop-start loop-end))
-			  (close-sound temp))
-			(sf2it (cdr lst)))))))
-	(sf2it (soundfont-info))))))
+      (let sf2it ((lst (soundfont-info)))
+	(if (pair? lst)
+	    (let* ((vals (car lst))
+		   (name (car vals))
+		   (start (cadr vals))
+		   (end (if (null? (cdr lst))
+			    (framples)
+			    (cadadr lst)))
+		   (loop-start (- (caddr vals) start))
+		   (loop-end (- (cadddr vals) start))
+		   (filename (string-append name ".aif")))
+	      (if (selection?)
+		  (set! (selection-member? #t) #f))
+	      (set! (selection-member?) #t)
+	      (set! (selection-position) start)
+	      (set! (selection-framples) (- end start))
+	      (save-selection filename (selection-srate) mus-bshort mus-aifc)
+	      (let ((temp (open-sound filename)))
+		(set! (sound-loop-info temp) (list loop-start loop-end))
+		(close-sound temp))
+	      (sf2it (cdr lst))))))))
 
 
 ;;; -------- open-next-file-in-directory
@@ -1836,8 +1820,7 @@ a sort of play list: (region-play-list (list (list reg0 0.0) (list reg1 0.5) (li
       (set! last-file-opened filename)
       (display last-file-opened)
       (let ((new-path (directory-from-path (file-name filename))))
-	(if (or (not (string? current-directory))
-		(not (string=? current-directory new-path)))
+	(if (not (equal? current-directory new-path))
 	    (get-current-files new-path)))
       #f)
     
@@ -1885,7 +1868,7 @@ a sort of play list: (region-play-list (list (list reg0 0.0) (list reg1 0.5) (li
       ;;        the calls are ordered out->in (or last first)
       ;; we take this list and create and evaluate a new function
       
-      (let ((dsp-chain (apply vector (reverse (map (lambda (gen)
+      (let ((dsp-chain (reverse (apply vector (map (lambda (gen)
 						     (if (pair? gen)
 							 (make-env gen :duration dur)
 							 gen))
@@ -1903,22 +1886,21 @@ a sort of play list: (region-play-list (list (list reg0 0.0) (list reg1 0.5) (li
 	    (let ((g (dsp-chain i))
 		  (gname (string->symbol (format #f "g~D" i))))
 	      (set! closure (cons `(,gname (dsp-chain ,i)) closure))
-	      (if (env? g)
-		  (set! body (if (eqv? body 0.0)
-				 `(env ,gname)
-				 `(* (env ,gname) ,body)))
-		  (if (readin? g)
-		      (set! body (if (eqv? body 0.0)
-				     `(readin ,gname)
-				     `(+ ,body (readin ,gname))))
-		      (if (mus-generator? g)
-			  (set! body (if (eqv? body 0.0)
-					 (list (string->symbol (mus-name g)) gname)
-					 (list (string->symbol (mus-name g)) gname body)))
-			  (set! body (list gname body)))))))
+	      (cond ((env? g)
+		     (set! body (if (eqv? body 0.0)
+				    `(env ,gname)
+				    `(* (env ,gname) ,body))))
+		    ((readin? g)
+		     (set! body (if (eqv? body 0.0)
+				    `(readin ,gname)
+				    `(+ ,body (readin ,gname)))))
+		    ((mus-generator? g)
+		     (set! body (if (eqv? body 0.0)
+				    (list (string->symbol (mus-name g)) gname)
+				    (list (string->symbol (mus-name g)) gname body))))
+		    (else (set! body (list gname body))))))
 	  
 	  ;; now patch the two together (the apply let below) and evaluate the resultant thunk
-	  
 	  (apply define (list 'inner)
 		 `((let ,closure
 		     (do ((k ,start (+ k 1)))
@@ -1948,14 +1930,13 @@ a sort of play list: (region-play-list (list (list reg0 0.0) (list reg1 0.5) (li
 passed as the arguments so to end with channel 3 in channel 0, 2 in 1, 0 in 2, and 1 in 3, (scramble-channels 3 2 0 1)"))
     (lambda new-order
       
-      (define (find-chan chans chan len)
-	(let ((pos #f))
-	  (do ((i 0 (+ i 1)))
-	      ((or pos (= i len)) pos)
-	    (if (= (chans i) chan)
-		(set! pos i)))))
-      
       (define (scramble-channels-1 cur-chans end-chans chans loc)
+	(define (find-chan chans chan len)
+	  (let ((pos #f))
+	    (do ((i 0 (+ i 1)))
+		((or pos (= i len)) pos)
+	      (if (= (chans i) chan)
+		  (set! pos i)))))
 	(if (> chans loc)
 	    (let* ((end-chan (end-chans loc)) ; we want this channel at loc
 		   (cur-chan (cur-chans loc)) ; this (original) channel is currently at loc
@@ -1995,12 +1976,12 @@ passed as the arguments so to end with channel 3 in channel 0, 2 in 1, 0 in 2, a
 		(reader (make-sampler)))
 	    (do ((i 0 (+ i 1)))
 		((= i len))
-	      (let ((y (next-sample reader)))
-		(let* ((sum-of-squares (moving-average buffer (* y y)))
-		       (now-silent (< sum-of-squares silence)))
-		  (if (not (eq? in-silence now-silent))
-		      (set! edges (cons i edges)))
-		  (set! in-silence now-silent)))))
+	      (let* ((y (next-sample reader))
+		     (sum-of-squares (moving-average buffer (* y y)))
+		     (now-silent (< sum-of-squares silence)))
+		(if (not (eq? in-silence now-silent))
+		    (set! edges (cons i edges)))
+		(set! in-silence now-silent))))
 	  (set! edges (append (reverse edges) (list (framples))))
 	  (let* ((len (length edges))
 		 (pieces (make-vector len #f))
@@ -2048,7 +2029,7 @@ passed as the arguments so to end with channel 3 in channel 0, 2 in 1, 0 in 2, a
   (let ((documentation "(reverse-by-blocks block-len snd chn): divide sound into block-len blocks, recombine blocks in reverse order"))
     (lambda* (block-len snd chn)
       (let* ((len (framples snd chn))
-	     (num-blocks (floor (/ len (* (srate snd) block-len)))))
+	     (num-blocks (floor (/ len (srate snd) block-len))))
 	(if (> num-blocks 1)
 	    (let* ((actual-block-len (ceiling (/ len num-blocks)))
 		   (rd (make-sampler (- len actual-block-len) snd chn))
@@ -2075,7 +2056,7 @@ passed as the arguments so to end with channel 3 in channel 0, 2 in 1, 0 in 2, a
   (let ((documentation "(reverse-within-blocks block-len snd chn): divide sound into blocks, recombine in order, but each block internally reversed"))
     (lambda* (block-len snd chn)
       (let* ((len (framples snd chn))
-	     (num-blocks (floor (/ len (* (srate snd) block-len)))))
+	     (num-blocks (floor (/ len (srate snd) block-len))))
 	(if (> num-blocks 1)
 	    (let ((actual-block-len (ceiling (/ len num-blocks)))
 		  (no-clicks-env (list 0.0 0.0  .01 1.0  .99 1.0  1.0 0.0)))
