@@ -2,9 +2,6 @@
 ;;;    ENVELOPES (env.scm)
 ;;; *************************
 
-(when (provided? 'pure-s7)
-  (define-macro (call-with-values producer consumer) `(,consumer (,producer))))
-
 
 ;;;=============================================================================
 ;;; Exponential envelopes
@@ -50,67 +47,65 @@
 	 (out-scaler (* 1.0 out-scaler))
 	 (ycutoff (and cutoff (expt base (+ offset (* cutoff scaler)))))
 	 (result ()))
+
     ;; linear interpolation
-    (letrec ((interpolate (lambda (xl yl xh yh xi) (+ yl (* (- xi xl) (/ (- yh yl) (- xh xl))))))
-	     ;;
-	     ;; recursively render one segment
-	     ;;   xl,xh   = x coordinates of segment ends
-	     ;;   yl,yh   = y coordinates of segment ends
-	     ;;   yle,yhe = exponential values of y coords of segment ends
-	     ;;   error   = linear domain error bound for rendering
-	     ;;
-	     (exp-seg (lambda (xl yle xh yhe yl yh error)
-			(let* ((xint (/ (+ xl xh) 2.0))
-			       (yint (interpolate xl yl xh yh xint))
-			       (yinte (interpolate xl yle xh yhe xint))
-			       (yexp (expt base yint))
-			       (yerr (- (expt base (+ yint error)) yexp)))
-			  ;; is the linear approximation accurate enough?
-			  ;; are we still over the cutoff limit?
-			  (if (and (> (abs (- yexp yinte)) yerr)
-				   (or (not ycutoff) (> yinte ycutoff)))
-			      ;; no --> add a breakpoint and recurse right and left
-			      (call-with-values
-				  (lambda () (exp-seg xl yle xint yexp yl yint error))
-				(lambda (xi yi)
-				  (call-with-values
-				      (lambda () (exp-seg xint yexp xh yhe yint yh error))
-				    (lambda (xj yj)
-				      (values (append xi (list xint) xj)
-					      (append yi (list yexp) yj))))))
-			      ;; yes --> don't need to add nu'ting to the envelope
-			      (values () ()))))))
-      ;; loop for each segment in the envelope
-      (let segs ((en env1))
-	(let* ((x (car en))
-	       (y (* 1.0 (cadr en)))
-	       (nx (caddr en))
-	       (ny (* 1.0 (cadddr en)))
-	       (yscl (+ offset (* y scaler)))
-	       (nyscl (+ offset (* ny scaler)))
-	       (xy (list x (if (or (not ycutoff)
-				   (>= (expt base yscl) ycutoff))
-			       (* out-scaler (expt base yscl))
-			       0.0))))
-	  (set! result (append result xy))
-	  (call-with-values
-	      (lambda () (exp-seg x (expt base yscl) nx (expt base nyscl) yscl nyscl error))
-	    (lambda (xs ys)
-	      (if (pair? ys)
-		  (let ((ys-scaled (map (lambda (y) (* y out-scaler)) ys)))
-		    (let vals ((xx xs) 
-			       (yy ys-scaled))
-		      (let ((x (car xx))
-			    (y (car yy)))
-			(set! result (append result (list x y)))
-			(if (> (length xx) 1)
-			    (vals (cdr xx) (cdr yy)))))))))
-	  (if (<= (length en) 4)
-	      (append result (list nx (if (or (not ycutoff)
-					      (>= (expt base nyscl) ycutoff))
-					  (* out-scaler (expt base nyscl))
-					  0.0)))
-	      (segs (cddr en))))))))
+    (define (interpolate xl yl xh yh xi)
+      (+ yl (* (- xi xl) (/ (- yh yl) (- xh xl)))))
+    
+    ;; recursively render one segment
+    ;;   xl,xh   = x coordinates of segment ends
+    ;;   yl,yh   = y coordinates of segment ends
+    ;;   yle,yhe = exponential values of y coords of segment ends
+    ;;   error   = linear domain error bound for rendering
+    (define (exp-seg xl yle xh yhe yl yh error)
+      (let* ((xint (/ (+ xl xh) 2.0))
+	     (yint (interpolate xl yl xh yh xint))
+	     (yinte (interpolate xl yle xh yhe xint))
+	     (yexp (expt base yint))
+	     (yerr (- (expt base (+ yint error)) yexp)))
+	;; is the linear approximation accurate enough?
+	;; are we still over the cutoff limit?
+	(if (not (and (> (abs (- yexp yinte)) yerr)
+		      (or (not ycutoff)
+			  (> yinte ycutoff))))
+	    ;; yes --> don't need to add nu'ting to the envelope
+	    (values () ())
+	    ;; no --> add a breakpoint and recurse right and left
+	    ((lambda (xi yi xj yj)
+	       (values (append xi (list xint) xj)
+		       (append yi (list yexp) yj)))
+	     (exp-seg xl yle xint yexp yl yint error)
+	     (exp-seg xint yexp xh yhe yint yh error)))))
+    
+    ;; loop for each segment in the envelope
+    (let segs ((en env1))
+      (let* ((x (car en))
+	     (y (* 1.0 (cadr en)))
+	     (nx (caddr en))
+	     (ny (* 1.0 (cadddr en)))
+	     (yscl (+ offset (* y scaler)))
+	     (nyscl (+ offset (* ny scaler)))
+	     (xy (list x (if (or (not ycutoff)
+				 (>= (expt base yscl) ycutoff))
+			     (* out-scaler (expt base yscl))
+			     0.0))))
+	(set! result (append result xy))
+	((lambda (xs ys)
+	   (if (pair? ys)
+	       (let vals ((xx xs)
+			  (yy (map (lambda (y) (* y out-scaler)) ys)))
+		 (let ((x (car xx))
+		       (y (car yy)))
+		   (set! result (append result (list x y)))
+		   (if (pair? (cdr xx))
+		       (vals (cdr xx) (cdr yy)))))))
+	 (exp-seg x (expt base yscl) nx (expt base nyscl) yscl nyscl error))
+	(if (<= (length en) 4)
+	    (append result (list nx (if (or (not ycutoff)
+					    (>= (expt base nyscl) ycutoff))
+					(* out-scaler (expt base nyscl))
+					0.0)))
+	    (segs (cddr en)))))))
 
 ;;; Amplitude envelope in dBs
 ;;;
@@ -475,124 +470,122 @@
 			(and (not (zero? grains))
 			     (>= grain-counter grains)))
 		    (set! happy #f))))
-	  (if happy
-	      (begin
-		;; back to the beginning of the grain
-		;(set! gr-offset 0)
-		;; start of grain in samples from beginning of note
-		(set! gr-from-beg (floor (- gr-start-sample beg)))
-		;; reset out-time dependent envelopes to current time
-		(set! (mus-location amp-env) gr-from-beg)
-		(set! (mus-location gr-dur) gr-from-beg)
-		(set! (mus-location gr-dur-spread) gr-from-beg)
-		(set! (mus-location sr-env) gr-from-beg)
-		(set! (mus-location sr-spread-env) gr-from-beg)
-		(set! (mus-location gr-start) gr-from-beg)
-		(set! (mus-location gr-start-spread) gr-from-beg)
-		(set! (mus-location gr-dens-env) gr-from-beg)
-		(set! (mus-location gr-dens-spread-env) gr-from-beg)
-		;; start of grain in input file
-		(set! in-start-value (+ (* (env gr-start) gr-start-scaler)
-					(mus-random (* 0.5 (env gr-start-spread)
-						       gr-start-scaler))))
-		(set! in-start (floor (* in-start-value in-file-sr)))
-		;; duration in seconds of the grain
-		(set! gr-duration (max grain-duration-limit
-				       (+ (env gr-dur)
-					  (mus-random (* 0.5 (env gr-dur-spread))))))
-		;; number of samples in the grain
-		(set! gr-samples (floor (* gr-duration *clm-srate*)))
-		;; new sample rate for grain
-		(set! gr-srate (if srate-linear
-				   (+ (env sr-env)
-				      (mus-random (* 0.5 (env sr-spread-env))))
-				   (* (env sr-env)
-				      (expt srate-base
-					    (mus-random (* 0.5 (env sr-spread-env)))))))
-		;; set new sampling rate conversion factor
-		(set! (mus-increment in-file-reader) gr-srate)
-		;; number of samples in input
-		(set! in-samples (floor (* gr-samples srate-ratio)))
-		
-		;; check for out of bounds condition in in-file pointers
-		(if (> (+ in-start in-samples) last-in-sample)
-		    (set! in-start (- last-in-sample in-samples))
-		    (if (< in-start 0)
-			(set! in-start 0)))
-		;; reset position of input file reader
-		(set! (mus-location rd) in-start)
-		
-		;; restart grain envelopes
-		(set! (mus-phase gr-env) 0.0)
-		(set! (mus-phase gr-env-end) 0.0)
-		;; reset grain envelope durations
-		(set! (mus-frequency gr-env) (/ gr-duration))
-		(set! (mus-frequency gr-env-end) (/ gr-duration))
-		;;
-		;; move position in output file for next grain
-		;;
-		(set! gr-dens (env gr-dens-env))
-		;; increment spread in output file for next grain
-		(set! gr-dens-spread (mus-random (* 0.5 (env gr-dens-spread-env))))
-		(set! grain-counter (+ grain-counter 1))
-		(set! where (cond (;; use duration of grains as delimiter
-				   (= where-to grani-to-grain-duration)
-				   gr-duration)
-				  (;; use start in input file as delimiter
-				   (= where-to grani-to-grain-start)
-				   in-start-value)
-				  (;; use sampling rate as delimiter
-				   (= where-to grani-to-grain-sample-rate)
-				   gr-srate)
-				  (;; use a random number as delimiter
-				   (= where-to grani-to-grain-random)
-				   (random 1.0))
-				  (else grani-to-locsig)))
-		(if (and where-bins
-			 (not (zero? where)))
-		    ;; set output scalers according to criteria
+	  (when happy
+	    ;; back to the beginning of the grain
+					;(set! gr-offset 0)
+	    ;; start of grain in samples from beginning of note
+	    (set! gr-from-beg (floor (- gr-start-sample beg)))
+	    ;; reset out-time dependent envelopes to current time
+	    (set! (mus-location amp-env) gr-from-beg)
+	    (set! (mus-location gr-dur) gr-from-beg)
+	    (set! (mus-location gr-dur-spread) gr-from-beg)
+	    (set! (mus-location sr-env) gr-from-beg)
+	    (set! (mus-location sr-spread-env) gr-from-beg)
+	    (set! (mus-location gr-start) gr-from-beg)
+	    (set! (mus-location gr-start-spread) gr-from-beg)
+	    (set! (mus-location gr-dens-env) gr-from-beg)
+	    (set! (mus-location gr-dens-spread-env) gr-from-beg)
+	    ;; start of grain in input file
+	    (set! in-start-value (+ (* (env gr-start) gr-start-scaler)
+				    (mus-random (* 0.5 (env gr-start-spread)
+						   gr-start-scaler))))
+	    (set! in-start (floor (* in-start-value in-file-sr)))
+	    ;; duration in seconds of the grain
+	    (set! gr-duration (max grain-duration-limit
+				   (+ (env gr-dur)
+				      (mus-random (* 0.5 (env gr-dur-spread))))))
+	    ;; number of samples in the grain
+	    (set! gr-samples (floor (* gr-duration *clm-srate*)))
+	    ;; new sample rate for grain
+	    (set! gr-srate (if srate-linear
+			       (+ (env sr-env)
+				  (mus-random (* 0.5 (env sr-spread-env))))
+			       (* (env sr-env)
+				  (expt srate-base
+					(mus-random (* 0.5 (env sr-spread-env)))))))
+	    ;; set new sampling rate conversion factor
+	    (set! (mus-increment in-file-reader) gr-srate)
+	    ;; number of samples in input
+	    (set! in-samples (floor (* gr-samples srate-ratio)))
+	    
+	    ;; check for out of bounds condition in in-file pointers
+	    (set! in-start (if (> (+ in-start in-samples) last-in-sample)
+			       (- last-in-sample in-samples)
+			       (max in-start 0)))
+	    ;; reset position of input file reader
+	    (set! (mus-location rd) in-start)
+	    
+	    ;; restart grain envelopes
+	    (set! (mus-phase gr-env) 0.0)
+	    (set! (mus-phase gr-env-end) 0.0)
+	    ;; reset grain envelope durations
+	    (set! (mus-frequency gr-env) (/ gr-duration))
+	    (set! (mus-frequency gr-env-end) (/ gr-duration))
+	    ;;
+	    ;; move position in output file for next grain
+	    ;;
+	    (set! gr-dens (env gr-dens-env))
+	    ;; increment spread in output file for next grain
+	    (set! gr-dens-spread (mus-random (* 0.5 (env gr-dens-spread-env))))
+	    (set! grain-counter (+ grain-counter 1))
+	    (set! where (cond (;; use duration of grains as delimiter
+			       (= where-to grani-to-grain-duration)
+			       gr-duration)
+			      (;; use start in input file as delimiter
+			       (= where-to grani-to-grain-start)
+			       in-start-value)
+			      (;; use sampling rate as delimiter
+			       (= where-to grani-to-grain-sample-rate)
+			       gr-srate)
+			      (;; use a random number as delimiter
+			       (= where-to grani-to-grain-random)
+			       (random 1.0))
+			      (else grani-to-locsig)))
+	    (if (and where-bins
+		     (not (zero? where)))
+		;; set output scalers according to criteria
+		(do ((chn 0 (+ chn 1)))
+		    ((or (= chn out-chans)
+			 (= chn where-bins-len)))
+		  (locsig-set! loc chn (if (< (where-bins chn)
+					      where
+					      (where-bins (+ chn 1)))
+					   1.0
+					   0.0)))
+		;; if not "where" see if the user wants to send to all channels
+		(if (= where-to grani-to-grain-allchans)
+		    ;; send the grain to all channels
 		    (do ((chn 0 (+ chn 1)))
-			((or (= chn out-chans)
-			     (= chn where-bins-len)))
-		      (locsig-set! loc chn (if (< (where-bins chn)
-						  where
-						  (where-bins (+ chn 1)))
-					       1.0
-					       0.0)))
-		    ;; if not "where" see if the user wants to send to all channels
-		    (if (= where-to grani-to-grain-allchans)
-			;; send the grain to all channels
-			(do ((chn 0 (+ chn 1)))
-			    ((= chn out-chans))
-			  (locsig-set! loc chn 1.0))
-			;; "where" is zero or unknown: use normal n-channel locsig, 
-			;; only understands mono reverb and 1, 2 or 4 channel output
-			(begin
-			  (set! (mus-location gr-dist) gr-from-beg)
-			  (set! (mus-location gr-dist-spread) gr-from-beg)
-			  (set! (mus-location gr-degree) gr-from-beg)
-			  (set! (mus-location gr-degree-spread) gr-from-beg)
-			  ;; set locsig parameters, for now only understands stereo
-			  (move-locsig loc
-				       (+ (env gr-degree)
-					  (mus-random (* 0.5 (env gr-degree-spread))))
-				       (+ (env gr-dist)
-					  (mus-random (* 0.5 (env gr-dist-spread))))))))
-
-		(let ((grend (+ gr-start-sample gr-samples)))
-		  (if interp-gr-envs
-		      (do ((gr-offset gr-start-sample (+ gr-offset 1)))
-			  ((= gr-offset grend))
-			(locsig loc gr-offset (* (env amp-env) 
-						 (src in-file-reader)
-						 (+ (* (env gr-int-env) (table-lookup gr-env-end))
-						    (* (env gr-int-env-1) (table-lookup gr-env))))))
-
-		      (do ((gr-offset gr-start-sample (+ gr-offset 1)))
-			  ((= gr-offset grend))
-			(locsig loc gr-offset (* (env amp-env) 
-						 (table-lookup gr-env)
-						 (src in-file-reader)))))))))))))
+			((= chn out-chans))
+		      (locsig-set! loc chn 1.0))
+		    ;; "where" is zero or unknown: use normal n-channel locsig, 
+		    ;; only understands mono reverb and 1, 2 or 4 channel output
+		    (begin
+		      (set! (mus-location gr-dist) gr-from-beg)
+		      (set! (mus-location gr-dist-spread) gr-from-beg)
+		      (set! (mus-location gr-degree) gr-from-beg)
+		      (set! (mus-location gr-degree-spread) gr-from-beg)
+		      ;; set locsig parameters, for now only understands stereo
+		      (move-locsig loc
+				   (+ (env gr-degree)
+				      (mus-random (* 0.5 (env gr-degree-spread))))
+				   (+ (env gr-dist)
+				      (mus-random (* 0.5 (env gr-dist-spread))))))))
+	    
+	    (let ((grend (+ gr-start-sample gr-samples)))
+	      (if interp-gr-envs
+		  (do ((gr-offset gr-start-sample (+ gr-offset 1)))
+		      ((= gr-offset grend))
+		    (locsig loc gr-offset (* (env amp-env) 
+					     (src in-file-reader)
+					     (+ (* (env gr-int-env) (table-lookup gr-env-end))
+						(* (env gr-int-env-1) (table-lookup gr-env))))))
+		  
+		  (do ((gr-offset gr-start-sample (+ gr-offset 1)))
+		      ((= gr-offset grend))
+		    (locsig loc gr-offset (* (env amp-env) 
+					     (table-lookup gr-env)
+					     (src in-file-reader))))))))))))
 
 
 ;; (with-sound (:channels 2 :reverb jc-reverb :reverb-channels 1) (let ((file "oboe.snd")) (grani 0 2 5 file :grain-envelope (raised-cosine))))
