@@ -5,18 +5,17 @@
 (provide 'libc.scm)
 
 ;; if loading from a different directory, pass that info to C
-(let ((current-file (port-filename (current-input-port))))
-  (let ((directory (and (or (char=? (current-file 0) #\/)
-			    (char=? (current-file 0) #\~))
-			(substring current-file 0 (- (length current-file) 9)))))
-    (when (and directory (not (member directory *load-path*)))
-      (set! *load-path* (cons directory *load-path*)))
-    (with-let (rootlet)
-      (require cload.scm))
-    (when (and directory (not (string-position directory *cload-cflags*)))
-      (set! *cload-cflags* (string-append "-I" directory " " *cload-cflags*)))))
+(let* ((current-file (port-filename (current-input-port)))
+       (directory (and (memv (current-file 0) '(#\/ #\~))
+		       (substring current-file 0 (- (length current-file) 9)))))
+  (when (and directory (not (member directory *load-path*)))
+    (set! *load-path* (cons directory *load-path*)))
+  (with-let (rootlet)
+    (require cload.scm))
+  (when (and directory (not (string-position directory *cload-cflags*)))
+    (set! *cload-cflags* (string-append "-I" directory " " *cload-cflags*))))
 
-(if (not (defined? '*libc*))
+(unless (defined? '*libc*)
     (define *libc*
       (with-let (unlet)
 	(set! *libraries* (cons (cons "libc.scm" (curlet)) *libraries*))
