@@ -24,16 +24,13 @@
 (define tree-for-each-reversed 
   (let ((documentation "(tree-for-each-reversed func tree) applies func to every leaf of 'tree' moving in reverse through all the lists"))
     (lambda (func tree)
-      (define (flatten lst)
-	;; there's probably a more elegant way to do this
-	(cond ((null? lst) ())
-	      ((pair? lst)
-	       (if (pair? (car lst))
-		   (append (flatten (car lst)) (flatten (cdr lst)))
-		   (cons (car lst) (flatten (cdr lst)))))
-	      (#t lst)))
-      (for-each func (reverse (flatten tree))))))
-
+      (for-each func
+		(reverse
+		 (let flatten ((lst tree))
+		   (cond ((null? lst) ())
+			 ((not (pair? lst)) lst)
+			 ((pair? (car lst)) (append (flatten (car lst)) (flatten (cdr lst))))
+			 (else (cons (car lst) (flatten (cdr lst)))))))))))
 
 (define mix-sound 
   (let ((documentation "(mix-sound file start) mixes file (all chans) at start in the currently selected sound."))
@@ -128,17 +125,17 @@ All mixes sync'd to it are also moved the same number of samples. (hook-remove m
 (define (mix-click-sets-amp)
   (hook-push mix-click-hook 
 	     (lambda (hook)
-	       (let ((n (hook 'id)))
-		 (let ((zeroed (mix-property :zero n)))
-		   (if (not zeroed)
-		       (begin
-			 (set! (mix-property :amp n) (mix-amp n))
-			 (set! (mix-amp n) 0.0)
-			 (set! (mix-property :zero n) #t))
-		       (begin
-			 (set! (mix-amp n) (mix-property :amp n))
-			 (set! (mix-property :zero n) #f)))
-		   (set! (hook 'result) #t))))))
+	       (let* ((n (hook 'id))
+		      (zeroed (mix-property :zero n)))
+		 (if zeroed
+		     (begin
+		       (set! (mix-amp n) (mix-property :amp n))
+		       (set! (mix-property :zero n) #f))
+		     (begin
+		       (set! (mix-property :amp n) (mix-amp n))
+		       (set! (mix-amp n) 0.0)
+		       (set! (mix-property :zero n) #t)))
+		 (set! (hook 'result) #t)))))
 
 					;(mix-click-sets-amp)
 
@@ -150,9 +147,9 @@ All mixes sync'd to it are also moved the same number of samples. (hook-remove m
     (lambda (n)
       (help-dialog "Mix Help"
 		   (format #f "Mix ~A (sync: ~A):~%  position: ~D = ~,3F secs~%  length: ~D (~,3F secs)~%  in: ~A[~D]~%  scaler: ~A~%  speed: ~A~%  env: ~A~A"
-			   (if (mix-name n)
-			       (format #f "~S (~A)" (mix-name n) n)
-			       (format #f "~A" n))
+			   (format #f (if (mix-name n)
+					  (values "~S (~A)" (mix-name n) n)
+					  (values "~A" n)))
 			   (mix-sync n)
 			   (mix-position n)
 			   (* 1.0 (/ (mix-position n) (srate (car (mix-home n)))))
@@ -342,9 +339,9 @@ last end of the mixes in 'mix-list'"))
 	      (let* ((beg-x (+ first-x (* x-scale (- (mix-position m) beg))))
 		     (end-x (+ first-x (* x-scale (- (+ (mix-position m) (framples m)) beg))))
 		     (wenv (window-envelope beg-x end-x overall-amp-env)))
-		(if (null? (mix-amp-env m))
-		    (set! (mix-amp-env m) wenv)
-		    (set! (mix-amp-env m) (multiply-envelopes (mix-amp-env m) wenv)))))
+		(set! (mix-amp-env m) (if (null? (mix-amp-env m)) 
+					  wenv
+					  (multiply-envelopes (mix-amp-env m) wenv)))))
 	    mix-list)))))))
 
 
@@ -553,27 +550,26 @@ begin time of each mix that starts after beg in the given channel"))
 		  (check-mix-tags snd i)))
 	      (let ((mxs (mixes snd chn))
 		    (changed #f))
-		(define (check-mix mx trailing-mixes)
-		  (if (pair? trailing-mixes)
-		      (let ((pos (mix-position mx))
-			    (ls (left-sample snd chn))
-			    (rs (right-sample snd chn)))
-			(if (<= ls pos rs)
-			    (let ((x (x->position (/ pos (srate snd))))
-				  (y (mix-tag-y mx)))
-			      (for-each 
-			       (lambda (other-mix)
-				 (let ((other-pos (mix-position other-mix)))
-				   (if (<= ls other-pos rs)
-				       (let ((other-x (x->position (/ other-pos (srate snd))))
-					     (other-y (mix-tag-y other-mix)))
-					 (if (and (< (abs (- x other-x)) 6)
-						  (< (abs (- y other-y)) 10))
-					     (begin
-					       (set! (mix-tag-y other-mix) (+ (mix-tag-y other-mix) 20))
-					       (set! changed #t)))))))
-			       trailing-mixes)))
-			(check-mix (car trailing-mixes) (cdr trailing-mixes)))))
-		(check-mix (car mxs) (cdr mxs))
+		(let check-mix ((mx (car mxs))
+				(trailing-mixes (cdr mxs)))
+		  (when (pair? trailing-mixes)
+		    (let ((pos (mix-position mx))
+			  (ls (left-sample snd chn))
+			  (rs (right-sample snd chn)))
+		      (when (<= ls pos rs)
+			(let ((x (x->position (/ pos (srate snd))))
+			      (y (mix-tag-y mx)))
+			  (for-each (lambda (other-mix)
+				      (let ((other-pos (mix-position other-mix)))
+					(if (<= ls other-pos rs)
+					    (let ((other-x (x->position (/ other-pos (srate snd))))
+						  (other-y (mix-tag-y other-mix)))
+					      (when (and (< (abs (- x other-x)) 6)
+							 (< (abs (- y other-y)) 10))
+						(set! (mix-tag-y other-mix) (+ (mix-tag-y other-mix) 20))
+						(set! changed #t))))))
+				    trailing-mixes)))
+		      (check-mix (car trailing-mixes) (cdr trailing-mixes)))))
 		(if changed
 		    (update-time-graph snd chn))))))))
+	      
\ No newline at end of file
