@@ -10,28 +10,23 @@
 (define make-polygon
   (lambda (args)
     (define (total-length vects len)
-      (if (null? vects)
-	  len
-	  (if (vector? (car vects))
-	      (total-length (cdr vects) (+ len (length (car vects))))
-	      (if (car vects)
-		  (total-length (cdr vects) (+ len 1))
-		  (total-length (cdr vects) len)))))
+      (cond ((null? vects)         len)
+	    ((vector? (car vects)) (total-length (cdr vects) (+ len (length (car vects)))))
+	    ((car vects)           (total-length (cdr vects) (+ len 1)))
+	    (else                  (total-length (cdr vects) len))))
     (define (set-vals vects start vals)
-      (if (null? vects)
-	  vals
-	  (if (vector? (car vects))
-	      (let* ((vect (car vects))
-		     (len (length vect)))
-		(do ((i 0 (+ 1 i)))
-		    ((= i len))
-		  (set! (vals (+ start i)) (vect i)))
-		(set-vals (cdr vects) (+ start len) vals))
-	      (if (car vects)
-		  (begin
-		    (set! (vals start) (car vects))
-		    (set-vals (cdr vects) (+ start 1) vals))
-		  (set-vals (cdr vects) start vals)))))
+      (cond ((null? vects) vals)
+	    ((vector? (car vects))
+	     (let* ((vect (car vects))
+		    (len (length vect)))
+	       (do ((i 0 (+ 1 i)))
+		   ((= i len))
+		 (set! (vals (+ start i)) (vect i)))
+	       (set-vals (cdr vects) (+ start len) vals)))
+	    ((car vects)
+	     (set! (vals start) (car vects))
+	     (set-vals (cdr vects) (+ start 1) vals))
+	    (else (set-vals (cdr vects) start vals))))
     (set-vals args 0 (make-vector (total-length args 0)))))
 
 (define (make-bezier-1 x0 y0 x1 y1 x2 y2 x3 y3 n)
@@ -41,8 +36,8 @@
 	 (cy (* 3 (- y1 y0)))
 	 (bx (- (* 3 (- x2 x1)) cx))
 	 (by (- (* 3 (- y2 y1)) cy))
-	 (ax (- x3 (+ x0 cx bx)))
-	 (ay (- y3 (+ y0 cy by)))
+	 (ax (- x3 x0 cx bx))
+	 (ay (- y3 y0 cy by))
 	 (incr (/ 1.0 n))
 	 (pts (make-vector (* 2 (+ n 1)))))
     (set! (pts 0) x0)
@@ -206,9 +201,9 @@
 		     ((8 9) 5) ; a-flat
 		     (else 6)))) ; b-flat
       (list pclass octave 
-	    (if (or (= pclass 1) (= pclass 6))
+	    (if (memv pclass '(1 6))
 		:sharp
-		(and (or (= pclass 3) (= pclass 8) (= pclass 10))
+		(and (memv pclass '(3 8 10)) 
 		     :flat))
 	    cclass
 	    pitch)))
@@ -262,15 +257,13 @@
 	      (draw-treble-clef x0 (+ y0 (* size .76)) size)
 	      (draw-bass-clef (+ x0 (* size .075)) (+ y0 (* size .26)) size))
 	  (set! x0 (+ x0 (* size .8))))
-	(if accidental
-	    (set! x0 (+ x0 (* size .1)))
-	    (set! x0 (+ x0 (* size .25)))))
+	(set! x0 (+ x0 (* size (if accidental .1 .25)))))
 
     ;; accidental
     (if accidental
 	(begin
 	  ((if (eq? accidental :sharp) draw-sharp draw-flat) x0 (+ y0 (* .02 size) (* line-sep 0.5 line)) size)
-	  (set! x0 (+ x0 (* .25 size)))))
+	  (set! x0 (+ x0 line-sep))))
 
     ;; notehead
     (set! notehead-y (+ y0 (* .02 size) (* line-sep 0.5 line)))
@@ -294,25 +287,26 @@
     
     ;; stem
     (if (< dur 3)
-	(if (> line 3)
-	    ;; stem up
-	    (fill-rectangle (+ x0 (* size .25)) (+ y0 (* .02 size) (* size -0.8) (* line-sep 0.5 line)) (* size .05) (* size 0.8))
-	    (fill-rectangle (- x0 (* size .02)) (+ y0 (* line-sep line 0.5)) (* size .05) (* size 0.8))))
+	(fill-rectangle
+	 (if (> line 3) ; stem up
+	     (values (+ x0 line-sep) (+ y0 (* 0.02 size) (* size -0.8) (* line-sep 0.5 line)))
+	     (values (- x0 (* size 0.02)) (+ y0 (* line-sep line 0.5))))
+	 (* size 0.05) (* size 0.8)))
 
     ;; flags
     (if (< dur .6)
 	(let ((base (+ y0 (* line-sep 0.5 line))))
 	  (if (> line 2)
-	      (draw-8th-flag-up (+ x0 (* size .25)) (+ base (* size -0.6)) size)
+	      (draw-8th-flag-up (+ x0 line-sep) (+ base (* size -0.6)) size)
 	      (draw-8th-flag-down x0 (+ base (* .7 size)) size))
 	  (if (< dur .3)
 	      (begin
 		(if (> line 2)
-		    (draw-extend-flag-up (+ x0 (* size .25)) (+ base (* size -0.8)) size)
+		    (draw-extend-flag-up (+ x0 line-sep) (+ base (* size -0.8)) size)
 		    (draw-extend-flag-down x0 (+ base (* .9 size)) size))
 		(if (< dur .15)
 		    (if (> line 2)
-			(draw-extend-flag-up (+ x0 (* size .25)) (+ base (* size -1.0)) size)
+			(draw-extend-flag-up (+ x0 line-sep) (+ base (* size -1.0)) size)
 			(draw-extend-flag-down x0 (+ base (* 1.1 size)) size)))))))
     (list notehead-x notehead-y)))
 
