@@ -39,11 +39,11 @@
 (define poly-as-vector-eval 
   (let ((documentation "(poly-as-vector-eval v x) treats 'v' as a vector of polynomial coefficients, returning the value of the polynomial at x"))
     (lambda (v x)
-      (let ((top (- (length v) 1)))
-	(let ((sum (v top)))
-	  (do ((i (- top 1) (- i 1)))
-	      ((< i 0) sum)
-	    (set! sum (+ (* sum x) (v i)))))))))
+      (let* ((top (- (length v) 1))
+	     (sum (v top)))
+	(do ((i (- top 1) (- i 1)))
+	    ((< i 0) sum)
+	  (set! sum (+ (* sum x) (v i))))))))
 
 
 (define poly-as-vector-reduce 
@@ -56,11 +56,7 @@
 			  (+ i 1)))))
 	(if (= new-len (length p1))
 	    p1
-	    (let ((np (make-vector new-len)))
-	      (do ((i 0 (+ i 1)))
-		  ((= i new-len))
-		(set! (np i) (p1 i)))
-	      np))))))
+	    (copy p1 (make-vector new-len)))))))
 
 (define poly-reduce 
   (let ((documentation "(poly-reduce p1) removes trailing (high-degree) zeros from the float-vector p1"))
@@ -105,8 +101,10 @@
 (define poly-as-vector* 
   (let ((documentation "(poly-as-vector* p1 p2) multiplies (as polynomials) the vectors p1 and p2"))
     (lambda (p1 p2)
-      (if (vector? p1)
-	  (if (vector? p2)
+      (if (not (vector? p1))
+	  (vector-scale! (copy p2) p1)
+	  (if (not (vector? p2))
+	      (vector-scale! (copy p1) p2)
 	      (let* ((p1len (length p1))
 		     (p2len (length p2))
 		     (len (+ p1len p2len))
@@ -116,9 +114,7 @@
 		  (do ((j 0 (+ j 1)))
 		      ((= j p2len))
 		    (set! (m (+ i j)) (+ (m (+ i j)) (* (p1 i) (p2 j))))))
-		m)
-	      (vector-scale! (copy p1) p2))
-	  (vector-scale! (copy p2) p1)))))
+		m))))))
 
 (define poly* 
   (let ((documentation "(poly* p1 p2) multiplies the polynomials (float-vectors or vectors) p1 and p2"))
@@ -138,8 +134,10 @@
 (define poly-as-vector/ 
   (let ((documentation "(poly-as-vector/ p1 p2) divides the polynomial p1 by p2 (both vectors)"))
     (lambda (p1 p2)
-      (if (vector? p1)
-	  (if (vector? p2)
+      (if (not (vector? p1))
+	  (list (vector 0) p2)
+	  (if (not (vector? p2))
+	      (list (poly-as-vector* p1 (/ p2)) (vector 0))
 	      ;; Numerical Recipes poldiv
 	      (let ((p1len (length p1))
 		    (p2len (length p2)))
@@ -162,9 +160,7 @@
 			(do ((j nv (+ j 1)))
 			    ((> j n))
 			  (set! (r j) 0))
-			(list q r)))))
-	      (list (poly-as-vector* p1 (/ p2)) (vector 0)))
-	  (list (vector 0) p2)))))
+			(list q r))))))))))
 
 (define poly/ 
   (let ((documentation "(poly/ p1 p2) divides p1 by p2, both polynomials either float-vectors or vectors"))
@@ -226,31 +222,29 @@
   (if (not (float-vector? mx))
       (error 'wrong-type-arg "determinant argument should be a float-vector")
       (let ((n (car (vector-dimensions mx))))
-	(if (= n 1) 
-	    (mx 0 0)
-	    (if (= n 2)
-		(- (* (mx 0 0) (mx 1 1))
-		   (* (mx 0 1) (mx 1 0)))
-		(if (= n 3)
-		    (- (+ (* (mx 0 0) (mx 1 1) (mx 2 2))
-			  (* (mx 0 1) (mx 1 2) (mx 2 0))
-			  (* (mx 0 2) (mx 1 0) (mx 2 1)))
-		       (+ (* (mx 0 0) (mx 1 2) (mx 2 1))
-			  (* (mx 0 1) (mx 1 0) (mx 2 2))
-			  (* (mx 0 2) (mx 1 1) (mx 2 0))))
-		    (let ((sum 0.0)
-			  (sign 1))
-		      (do ((i 0 (+ i 1)))
-			  ((= i n))
-			(let ((mult (mx 0 i)))
-			  (if (not (= mult 0.0))
-			      (set! sum (+ sum (* sign mult (determinant (submatrix mx 0 i))))))
-			  (set! sign (- sign))))
-		      sum)))))))
+	(case n
+	  ((1) (mx 0 0))
+	  ((2) (- (* (mx 0 0) (mx 1 1)) (* (mx 0 1) (mx 1 0))))
+	  ((3) (- (+ (* (mx 0 0) (mx 1 1) (mx 2 2))
+		     (* (mx 0 1) (mx 1 2) (mx 2 0))
+		     (* (mx 0 2) (mx 1 0) (mx 2 1)))
+		  (* (mx 0 0) (mx 1 2) (mx 2 1))
+		  (* (mx 0 1) (mx 1 0) (mx 2 2))
+		  (* (mx 0 2) (mx 1 1) (mx 2 0))))
+	  (else   
+	   (let ((sum 0.0)
+		 (sign 1))
+	     (do ((i 0 (+ i 1)))
+		 ((= i n))
+	       (let ((mult (mx 0 i)))
+		 (if (not (= mult 0.0))
+		     (set! sum (+ sum (* sign mult (determinant (submatrix mx 0 i))))))
+		 (set! sign (- sign))))
+	     sum))))))
 
 (define (poly-as-vector-resultant p1 p2)
-  (if (or (not (vector? p1)) 
-	  (not (vector? p2)))
+  (if (not (and (vector? p1)
+		(vector? p2)))
       (error 'wrong-type-arg "poly-as-vector-resultant arguments should be vectors")
       (let* ((m (length p1))
 	     (n (length p2))
@@ -359,8 +353,8 @@
   
   (define (quadratic-roots a b c) ; ax^2 + bx + c
     (let ((d (sqrt (- (* b b) (* 4 a c)))))
-      (list (/ (+ (- b) d) (* 2 a))
-	    (/ (- (- b) d) (* 2 a)))))
+      (list (/ (- d b) (* 2 a))
+	    (/ (- (+ d b)) (* 2 a)))))
   
   (define (cubic-roots a b c d) ; ax^3 + bx^2 + cx + d
     ;; Abramowitz & Stegun 3.8.2
@@ -403,32 +397,30 @@
 	      (a1 (/ d a))
 	      (a2 (/ c a))
 	      (a3 (/ b a))
-	      (yroot (poly-as-vector-roots (vector (+ (* 4 a2 a0) (- (* a1 a1)) (- (* a3 a3 a0)))
+	      (yroot (poly-as-vector-roots (vector (- (* 4 a2 a0) (* a1 a1) (* a3 a3 a0))
 						   (- (* a1 a3) (* 4 a0))
 						   (- a2)
 						   1.0))))
-	 (if (and yroot
-		  (pair? yroot)
-		  (= (length yroot) 4))
-	     (do ((i 0 (+ i 1)))
-		 ((= i 3))
-	       (let* ((y1 (yroot i))
-		      (R (sqrt (+ (* 0.25 a3 a3) (- a2) y1)))
-		      (D (if (= R 0)
-			     (sqrt (+ (* 0.75 a3 a3) (* -2 a2) (* 2 (sqrt (- (* y1 y1) (* 4 a0))))))
-			     (sqrt (+ (* 0.75 a3 a3) (* -2 a2) (- (* R R))
-				      (/ (* 0.25 (+ (* 4 a3 a2) (* -8 a1) (- (* a3 a3 a3)))) R)))))
-		      (E (if (= R 0)
-			     (sqrt (+ (* 0.75 a3 a3) (* -2 a2) (* -2 (sqrt (- (* y1 y1) (* 4 a0))))))
-			     (sqrt (+ (* 0.75 a3 a3) (* -2 a2) (- (* R R)) 
-				      (/ (* -0.25 (+ (* 4 a3 a2) (* -8 a1) (- (* a3 a3 a3)))) R)))))
-		      (z1 (+ (* -0.25 a3) (* 0.5 R) (* 0.5 D)))
-		      (z2 (+ (* -0.25 a3) (* 0.5 R) (* -0.5 D)))
-		      (z3 (+ (* -0.25 a3) (* -0.5 R) (* 0.5 E)))
-		      (z4 (+ (* -0.25 a3) (* -0.5 R) (* -0.5 E))))
-		 
-		 (if (< (magnitude (poly-as-vector-eval (vector e d c b a) z1)) poly-roots-epsilon)
-		     (return (list z1 z2 z3 z4))))))
+	 (when (and yroot
+		    (pair? yroot)
+		    (= (length yroot) 4))
+	   (do ((i 0 (+ i 1)))
+	       ((= i 3))
+	     (let* ((y1 (yroot i))
+		    (R (sqrt (- (+ (* 0.25 a3 a3) y1) a2)))
+		    (D (if (= R 0)
+			   (sqrt (+ (* 0.75 a3 a3) (* -2 a2) (* 2 (sqrt (- (* y1 y1) (* 4 a0))))))
+			   (sqrt (- (+ (* 0.75 a3 a3) (* -2 a2) (/ (* 0.25 (- (+ (* 4 a3 a2) (* -8 a1)) (* a3 a3 a3))) R)) (* R R)))))
+		    (E (if (= R 0)
+			   (sqrt (+ (* 0.75 a3 a3) (* -2 a2) (* -2 (sqrt (- (* y1 y1) (* 4 a0))))))
+			   (sqrt (- (+ (* 0.75 a3 a3) (* -2 a2) (/ (* -0.25 (- (+ (* 4 a3 a2) (* -8 a1)) (* a3 a3 a3))) R)) (* R R)))))
+		    (z1 (+ (* -0.25 a3) (* 0.5 R) (* 0.5 D)))
+		    (z2 (+ (* -0.25 a3) (* 0.5 R) (* -0.5 D)))
+		    (z3 (+ (* -0.25 a3) (* -0.5 R) (* 0.5 E)))
+		    (z4 (+ (* -0.25 a3) (* -0.5 R) (* -0.5 E))))
+	       
+	       (if (< (magnitude (poly-as-vector-eval (vector e d c b a) z1)) poly-roots-epsilon)
+		   (return (list z1 z2 z3 z4))))))
 	 #f))))
   
   (define (nth-roots a b deg) ; ax^n + b
@@ -442,120 +434,121 @@
   
   (let ((deg (- (length p1) 1)))
     
-    (if (= deg 0)                                     ; just constant
-	()
+    (cond ((= deg 0)                               ; just constant
+	   ())
 	
-	(if (= (p1 0) 0.0)                 ; constant=0.0, divide through by x, recurse on new
-	    (if (= deg 1)
-		(list 0.0)
-		(let ((pnew (make-vector deg)))
-		  (do ((i 1 (+ i 1)))
-		      ((> i deg))
-		    (set! (pnew (- i 1)) (p1 i)))
-		  (append (list 0.0) (poly-as-vector-roots pnew))))
+	  ((= (p1 0) 0.0)                          ; constant=0.0, divide through by x, recurse on new
+	   (if (= deg 1)
+	       (list 0.0)
+	       (let ((pnew (make-vector deg)))
+		 (do ((i 1 (+ i 1)))
+		     ((> i deg))
+		   (set! (pnew (- i 1)) (p1 i)))
+		 (cons 0.0 (poly-as-vector-roots pnew)))))
 	    
-	    (if (= deg 1)                             ; ax + b -> -b/a
-		(linear-root (p1 1) (p1 0))
+	    ((= deg 1)                             ; ax + b -> -b/a
+	     (linear-root (p1 1) (p1 0)))
 		
-		(if (= deg 2)                         ; ax^2 + bx + c -> -b +/- sqrt(b^2 - 4ac) / 2a
-		    (quadratic-roots (p1 2) (p1 1) (p1 0))
-		    
-		    (or (and (= deg 3)
-			     ;; it may be better to fall into Newton's method here
-			     (cubic-roots (p1 3) (p1 2) (p1 1) (p1 0)))
-			
-			(and (= deg 4)
-			     (quartic-roots (p1 4) (p1 3) (p1 2) (p1 1) (p1 0)))
-			
-			;; degree>4 (or trouble above), use Newton's method unless some simple case pops up
-			(let ((ones 0))
-			  (do ((i 1 (+ i 1)))
-			      ((> i deg))
-			    (if (not (= (p1 i) 0.0))
-				(set! ones (+ 1 ones))))
-			  
-			  (if (= ones 1)                  ; x^n + b -- "linear" in x^n
-			      (nth-roots (p1 deg) (p1 0) deg)
-			      
-			      (if (and (= ones 2)
-				       (even? deg)
-				       (not (= (p1 (/ deg 2)) 0.0)))
-				  (let ((roots ())       ; quadratic in x^(n/2)
-					(n (/ deg 2)))
-				    (for-each
-				     (lambda (r)
-				       (set! roots (append roots (nth-roots 1.0 (- r) n))))
-				     (poly-as-vector-roots (vector (p1 0) 
-								   (p1 (/ deg 2)) 
-								   (p1 deg))))
-				    roots)
-				  
-				  (if (and (> deg 3)
-					   (= ones 3)
-					   (= (modulo deg 3) 0)
-					   (not (= (p1 (/ deg 3)) 0.0))
-					   (not (= (p1 (/ (* 2 deg) 3)) 0.0)))
-				      (let ((roots ())   ; cubic in x^(n/3)
-					    (n (/ deg 3)))
-					(for-each
-					 (lambda (r)
-					   (set! roots (append roots (nth-roots 1.0 (- r) n))))
-					 (poly-as-vector-roots (vector (p1 0) 
-								       (p1 (/ deg 3)) 
-								       (p1 (/ (* 2 deg) 3))
-								       (p1 deg))))
-					roots)
-				      
-				      ;; perhaps get derivative roots, plug in main -- need to get nth derivative to be safe in this
-				      ;; from Cohen, "Computational Algebraic Number Theory"
-				      (let* ((roots ())
-					     (q (copy p1))
-					     (pp (poly-as-vector-derivative p1))
-					     (qp (copy pp))
-					     (n deg)
-					     (x 1.3+0.314159i)
-					     (v (poly-as-vector-eval q x))
-					     (m (* (magnitude v) (magnitude v)))
-					     (dx 0.0)
-					     (last-dx 1.0) ; guard against infinite loop
-					     (happy #f))
-					(do ()
-					    (happy)
-					  (set! dx (/ v (poly-as-vector-eval qp x)))
-					  (if (or (<= (magnitude dx) poly-roots-epsilon)
-						  (= dx last-dx))
-					      (set! happy #t)
+	    ((= deg 2)                             ; ax^2 + bx + c -> -b +/- sqrt(b^2 - 4ac) / 2a
+	     (quadratic-roots (p1 2) (p1 1) (p1 0)))
+
+	    (else
+	     (or (and (= deg 3)
+		     ;; it may be better to fall into Newton's method here
+		     (cubic-roots (p1 3) (p1 2) (p1 1) (p1 0)))
+		 
+		 (and (= deg 4)
+		      (quartic-roots (p1 4) (p1 3) (p1 2) (p1 1) (p1 0)))
+		 
+		 ;; degree>4 (or trouble above), use Newton's method unless some simple case pops up
+		 (let ((ones 0))
+		   (do ((i 1 (+ i 1)))
+		       ((> i deg))
+		     (if (not (= (p1 i) 0.0))
+			 (set! ones (+ 1 ones))))
+		   
+		   (cond ((= ones 1)                  ; x^n + b -- "linear" in x^n
+			  (nth-roots (p1 deg) (p1 0) deg))
+		       
+		       ((and (= ones 2)
+			     (even? deg)
+			     (not (= (p1 (/ deg 2)) 0.0)))
+			(let ((roots ())       ; quadratic in x^(n/2)
+			      (n (/ deg 2)))
+			  (for-each
+			   (lambda (r)
+			     (set! roots (append roots (nth-roots 1.0 (- r) n))))
+			   (poly-as-vector-roots (vector (p1 0) 
+							 (p1 (/ deg 2)) 
+							 (p1 deg))))
+			  roots))
+			   
+		       ((and (> deg 3)
+			     (= ones 3)
+			     (= (modulo deg 3) 0)
+			     (not (= (p1 (/ deg 3)) 0.0))
+			     (not (= (p1 (/ (* 2 deg) 3)) 0.0)))
+			(let ((roots ())   ; cubic in x^(n/3)
+			      (n (/ deg 3)))
+			  (for-each
+			   (lambda (r)
+			     (set! roots (append roots (nth-roots 1.0 (- r) n))))
+			   (poly-as-vector-roots (vector (p1 0) 
+							 (p1 (/ deg 3)) 
+							 (p1 (/ (* 2 deg) 3))
+							 (p1 deg))))
+			  roots))
+
+		       (else 
+			;; perhaps get derivative roots, plug in main -- need to get nth derivative to be safe in this
+			;; from Cohen, "Computational Algebraic Number Theory"
+			(let ((roots ())
+			      (q (copy p1))
+			      (n deg)
+			      (x 1.3+0.314159i))
+			  (let ((pp (poly-as-vector-derivative p1)))
+			    (let ((happy #f)
+				  (qp (copy pp))
+				  (dx 0.0)
+				  (v (poly-as-vector-eval q x))
+				  (last-dx 1.0)) ; guard against infinite loop
+			      (do ((m (* (magnitude v) (magnitude v))))
+				  (happy)
+				(set! dx (/ v (poly-as-vector-eval qp x)))
+				(if (or (<= (magnitude dx) poly-roots-epsilon)
+					(= dx last-dx))
+				    (set! happy #t)
+				    (begin
+				      (set! last-dx dx)
+				      (do ((c 0 (+ 1 c))
+					   (step3 #f))
+					  ((or (>= c 20)
+					       step3
+					       (<= (magnitude dx) poly-roots-epsilon)))
+					(let* ((y (- x dx))
+					       (v1 (poly-as-vector-eval q y))
+					       (m1 (* (magnitude v1) (magnitude v1))))
+					  (if (< m1 m)
 					      (begin
-						(set! last-dx dx)
-						(do ((c 0 (+ 1 c))
-						     (step3 #f))
-						    ((or (>= c 20)
-							 step3
-							 (<= (magnitude dx) poly-roots-epsilon)))
-						  (let* ((y (- x dx))
-							 (v1 (poly-as-vector-eval q y))
-							 (m1 (* (magnitude v1) (magnitude v1))))
-						    (if (< m1 m)
-							(begin
-							  (set! x y)
-							  (set! v v1)
-							  (set! m m1)
-							  (set! step3 #t))
-							(set! dx (/ dx 4.0))))))))
-					(set! x (- x (/ (poly-as-vector-eval p1 x) (poly-as-vector-eval pp x))))
-					(set! x (- x (/ (poly-as-vector-eval p1 x) (poly-as-vector-eval pp x))))
-					(if (< (imag-part x) poly-roots-epsilon)
-					    (begin
-					      (set! x (real-part x))
-					      (set! q (poly-as-vector/ q (vector (- x) 1.0)))
-					      (set! n (- n 1)))
-					    (begin
-					      (set! q (poly-as-vector/ q (vector (magnitude x) 0.0 1.0)))
-					      (set! n (- n 2))))
-					(set! roots (cons x roots))
-					(if (> n 0) 
-					    (set! roots (append (poly-as-vector-roots (poly-as-vector-reduce (car q))) roots)))
-					roots))))))))))))
+						(set! x y)
+						(set! v v1)
+						(set! m m1)
+						(set! step3 #t))
+					      (set! dx (/ dx 4.0)))))))))
+			    (set! x (- x (/ (poly-as-vector-eval p1 x) (poly-as-vector-eval pp x))))
+			    (set! x (- x (/ (poly-as-vector-eval p1 x) (poly-as-vector-eval pp x)))))
+			  (if (< (imag-part x) poly-roots-epsilon)
+			      (begin
+				(set! x (real-part x))
+				(set! q (poly-as-vector/ q (vector (- x) 1.0)))
+				(set! n (- n 1)))
+			      (begin
+				(set! q (poly-as-vector/ q (vector (magnitude x) 0.0 1.0)))
+				(set! n (- n 2))))
+			  (set! roots (cons x roots))
+			  (if (> n 0) 
+			      (set! roots (append (poly-as-vector-roots (poly-as-vector-reduce (car q))) roots)))
+			  roots)))))))))
 
 (define poly-roots 
   (let ((documentation "(poly-roots p1) returns the roots of polynomial p1"))
@@ -565,7 +558,8 @@
 	(for-each
 	 (lambda (q)
 	   (let ((dx (magnitude (poly-as-vector-eval v1 q))))
-	     (if (> dx poly-roots-epsilon) (format #t ";poly.scm 502: (poly-roots ~A) numerical trouble (polynomial root is not very good): ~A at ~A: ~A" p1 v1 q dx))))
+	     (if (> dx poly-roots-epsilon) 
+		 (format () ";poly.scm 502: (poly-roots ~A) numerical trouble (polynomial root is not very good): ~A at ~A: ~A" p1 v1 q dx))))
 	 roots)
 	roots))))
 
