@@ -19,15 +19,8 @@
     (copy s (make-vector (- stop start)) start stop)))
 
 (define list-copy copy)
-
-(define* (vector-copy v (start 0) end)
-  (let ((stop (or end (length v)))) 
-    (copy v (make-vector (- stop start)) start stop)))
-
-(define* (r7rs-string-copy s (start 0) end) 
-  (let ((stop (or end (length s)))) 
-    (copy s (make-string (- stop start)) start stop)))
-
+(define vector-copy string->vector)
+(define r7rs-string-copy vector->string) ; the latter doesn't know its not a vector
 (define r7rs-vector-fill! fill!) ; or do these return the sequence, not the filler?
 (define r7rs-string-fill! fill!)
 
@@ -133,7 +126,7 @@
 (define u8-ready? char-ready?) 
 (define peek-u8 peek-char)
 (define* (utf8->string v (start 0) end) (substring v start (or end (length v))))
-(define* (string->utf8 s (start 0) end) (->byte-vector (substring s start (or end (length s)))))
+(define* (string->utf8 s (start 0) end) (->byte-vector (utf8->string s start end)))
 (define write-simple write)
 
 (define (eof-object) #<eof>)
@@ -166,16 +159,56 @@
 ;;   or (format #f "~^ this is a comment ")
 
 
-(define-macro (define-values vars . body)
-  `(apply begin (map (lambda (var val) `(define ,var ,val)) ',vars (list (begin ,@body)))))
+(define-macro (define-values vars expression)
+  `(if (not (null? ',vars))
+       (varlet (curlet) ((lambda ,vars (curlet)) ,expression))))
 
-(define-macro (let*-values vars . body)
-  `(let () 
-     ,@(map (lambda (nvars . nbody)
-              `(apply define-values ',nvars ',@nbody))
-            (map car vars) 
-            (map cdr vars))
-     ,@body))
+#|
+(define-macro (define*-values vars expression) ; same but allows defaults for the vars
+  `(if (not (null? ',vars))
+       (varlet (curlet) ((lambda* ,vars (curlet)) ,expression))))
+
+(define-macro (define-values vars . body) ; but the spec says "<expression>" here
+  `(apply begin (map (lambda (var val) `(define ,var ,val)) ',vars (list (begin ,@body)))))
+|#
+
+(define-macro (let-values vars . body)
+  (if (and (pair? vars)
+	   (pair? (car vars))
+	   (null? (cdar vars)))
+      `((lambda ,(caar vars)
+	  ,@body)
+	,(cadar vars))
+      `(with-let (apply sublet (curlet)
+			(list ,@(map (lambda (v)
+				       `((lambda ,(car v)
+					   (values ,@(map (lambda (name)
+							    (values (symbol->keyword name) name))
+							  (let args->proper-list ((args (car v)))
+							    (cond ((symbol? args)	(list args))
+								  ((not (pair? args))	args)
+								  ((pair? (car args))	(cons (caar args) (args->proper-list (cdr args))))
+								  (else                 (cons (car args) (args->proper-list (cdr args)))))))))
+					 ,(cadr v)))
+				     vars)))
+	 ,@body)))
+
+(define-macro (let*-values vals . body)
+  (let ((args ())
+	(exprs ()))
+    (for-each
+     (lambda (arg+expr)
+       (set! args (cons (car arg+expr) args))
+       (set! exprs (cons (cadr arg+expr) exprs)))
+     vals)
+    (let ((form `((lambda ,(car args) ,@body) ,(car exprs))))
+      (if (not (null? (cdr args)))
+	  (for-each
+	   (lambda (arg expr)
+	     (set! form `((lambda ,arg ,form) ,expr)))
+	   (cdr args)
+	   (cdr exprs)))
+      form)))
 
 
 ;; case-lambda       
@@ -296,7 +329,7 @@
 		(else
 		 `(let ((sym (symbol (object->string ',lib))))
 		    (if (not (defined? sym))
-			(format #t "~A not loaded~%" sym)
+			(format () "~A not loaded~%" sym)
 			(symbol->value sym))))))
 	    libs)))
 
@@ -388,9 +421,9 @@
 ;; records
 (define-macro (define-record-type type make ? . fields)
   (let ((new-type (if (pair? type) (car type) type))
-	(inherited (if (pair? type) `(list ,@(cdr type)) ())))
+	(inherited (if (pair? type) (cdr type) ())))
     `(begin
-       (define-class ,new-type ,inherited
+       (define-class ,new-type ,inherited   ; from stuff.scm
          (map (lambda (f) (if (pair? f) (car f) f)) ',fields))
        
        (define (,? obj)    ; perhaps the define-class type predicate should use this 
@@ -413,23 +446,23 @@
        
        ,@(map
 	  (lambda (field)
-	    (if (pair? field)
-	        (if (null? (cdr field))
-		    (values)
-		    (if (null? (cddr field))
-		        `(define (,(cadr field) obj)
-                           (if (not (,? obj)) 
-                               (error 'wrong-type-arg "~S should be a ~A" obj ',type))
-                           (obj ',(car field)))
-		        `(begin
-			   (define (,(cadr field) obj)
-			     (if (not (,? obj)) 
-				 (error 'wrong-type-arg "~S should be a ~A" obj ',type))
-                             (obj ',(car field)))
-			   (define (,(caddr field) obj val)
-			     (if (not (,? obj)) 
-				 (error 'wrong-type-arg "~S should be a ~A" obj ',type))
-                             (set! (obj ',(car field)) val)))))))
+	    (when (pair? field)
+	      (if (null? (cdr field))
+		  (values)
+		  (if (null? (cddr field))
+		      `(define (,(cadr field) obj)
+			 (if (not (,? obj)) 
+			     (error 'wrong-type-arg "~S should be a ~A" obj ',type))
+			 (obj ',(car field)))
+		      `(begin
+			 (define (,(cadr field) obj)
+			   (if (not (,? obj)) 
+			       (error 'wrong-type-arg "~S should be a ~A" obj ',type))
+			   (obj ',(car field)))
+			 (define (,(caddr field) obj val)
+			   (if (not (,? obj)) 
+			       (error 'wrong-type-arg "~S should be a ~A" obj ',type))
+			   (set! (obj ',(car field)) val)))))))
 	  fields)
        ',new-type)))
 
