@@ -66,17 +66,18 @@
 	      (histsize 100)
 	      (histpos 0)
 	      (m-p-pos 0)
-	      (histtop ())
-	      (histtail ()))
+	      (histtop ()))
 
-	  (define (push-line line)
-	    (if (null? histtop)
-		(begin
-		  (set! histtop (list line))
-		  (set! histtail histtop))
-		(begin
-		  (set-cdr! histtail (list line))
-		  (set! histtail (cdr histtail)))))
+	  (define push-line 
+	    (let ((histtail ()))
+	      (lambda (line)
+		(if (null? histtop)
+		    (begin
+		      (set! histtop (list (copy line)))
+		      (set! histtail histtop))
+		    (begin
+		      (set-cdr! histtail (list line))
+		      (set! histtail (cdr histtail)))))))
 
 	  (define (history-member line)
 	    (do ((i 0 (+ i 1)))
@@ -96,13 +97,9 @@
 					  (let loop ((oldpos histpos)
 						     (newpos new-end))
 					    (set! (new-hist newpos) (histbuf oldpos))
-					    (if (zero? newpos)
-						(set! newpos (- new-size 1))
-						(set! newpos (- newpos 1)))
+					    (set! newpos (- (if (zero? newpos) new-size newpos) 1))
 					    (unless (= newpos new-end)
-					      (if (zero? oldpos)
-						  (set! oldpos (- histsize 1))
-						  (set! oldpos (- oldpos 1)))
+					      (set! oldpos (- (if (zero? oldpos) histsize oldpos) 1))
 					      (unless (= oldpos histpos)
 						(loop oldpos newpos))))
 					  (set! histsize new-size)
@@ -113,11 +110,11 @@
 	  (define history (dilambda 
 			   (lambda (back)
 			     (let ((i (+ histpos back)))
-			       (if (< i 0)
-				   (histbuf (+ histsize i))
-				   (if (>= i histsize)
-				       (histbuf (- i histsize))
-				       (histbuf i)))))
+			       (copy (if (< i 0)
+					 (histbuf (+ histsize i))
+					 (if (>= i histsize)
+					     (histbuf (- i histsize))
+					     (histbuf i))))))
 
 			   (lambda (new-line)
 			     (let ((pos (history-member new-line)))
@@ -128,16 +125,15 @@
 				     (set! (histbuf i) (histbuf (+ i 1))))
 				   (set! (histbuf (- histsize 1)) (histbuf 0))
 				   (set! pos 0))
-
 				 (do ((i pos (+ i 1)))
 				     ((>= i (- histpos 1)))
 				   (set! (histbuf i) (histbuf (+ i 1))))
-				 (set! histpos (- histpos 1)))
+				 (set! histpos (- histpos 1))))
 
-			       (set! (histbuf histpos) new-line)
-			       (set! histpos (+ histpos 1))
-			       (if (= histpos histsize)
-				   (set! histpos 0))))))
+			     (set! (histbuf histpos) (copy new-line))
+			     (set! histpos (+ histpos 1))
+			     (if (= histpos histsize)
+				 (set! histpos 0)))))
 	  
 	  (define (history-help)
 	    (set! (*repl* 'helpers)
@@ -193,7 +189,10 @@
 	    
 	    ;; -------- evaluation ---------
 	    (define (badexpr h)            ; *missing-close-paren-hook* function for Enter command
-	      (set! (h 'result) 'string-read-error))
+	      (let ((ow (owlet)))
+		(if (ow 'error-file)
+		    (error "missing close paren in ~S" (ow 'error-file))
+		    (set! (h 'result) 'string-read-error))))
 	    
 	    (define (shell? h)             ; *unbound-variable-hook* function, also for Enter
 	      ;; examine cur-line -- only call system if the unbound variable matches the first non-whitespace chars
@@ -267,16 +266,19 @@
 	    
 	    
 	    ;; -------- match parens --------
-	    (define (char-constant? pos)
-	      (and (> pos 2)
-		   (char=? (cur-line (- pos 1)) #\\)
-		   (char=? (cur-line (- pos 2)) #\#)))
-	    
 	    (define (check-parens)
+
+	      (define (char-constant? pos)
+		(and (> pos 2)
+		     (char=? (cur-line (- pos 1)) #\\)
+		     (char=? (cur-line (- pos 2)) #\#)))
+	    
 	      (let ((endpos (- cursor-pos 1)))
-		(if (and (> cursor-pos 1)
-			 (char=? (cur-line endpos) #\))              ; ")" on left of cursor
-			 (not (char-constant? endpos)))                  ; it's not "#\)"
+		(if (or (<= cursor-pos 1)
+			(not (char=? (cur-line endpos) #\)))      ; ")" on left of cursor
+			(char-constant? endpos))                  ; it's not "#\)"
+		    (if (number? red-par-pos)
+			(set! red-par-pos #f))
 		    (let ((oparens ())
 			  (new-red-pos #f))
 		      (do ((i 0 (+ i 1)))
@@ -315,15 +317,10 @@
 				    (set! i (+ k 1))
 				    (if (>= i endpos)
 					(set! oparens ()))))))))
-		      
 		      (if (pair? oparens)
 			  (set! new-red-pos (car oparens)))
 		      (unless (equal? new-red-pos red-par-pos)
-			(if (number? new-red-pos)
-			    (set! red-par-pos new-red-pos)
-			    (set! red-par-pos #f))))
-		    (if (number? red-par-pos)
-			(set! red-par-pos #f)))))
+			(set! red-par-pos (and (number? new-red-pos) new-red-pos)))))))
 	    
 	    
 	    ;; -------- indentation --------
@@ -336,31 +333,29 @@
 		(check-parens)
 		(set! cur-line old-line)
 		(set! cursor-pos old-cursor)
+		(when red-par-pos
 		(let ((new-red red-par-pos))
 		  (set! red-par-pos old-red)
 		  (let ((col 0))
 		    (do ((i 0 (+ i 1)))
 			((= i new-red))
-		      (if (char=? (cur-line i) #\newline)
-			  (set! col 0)
-			  (set! col (+ col 1))))
-		    (let ((sym (do ((i (+ new-red 1) (+ i 1)))
-				   ((not (char-alphabetic? (cur-line i)))
-				    (substring cur-line (+ new-red 1) i)))))
-		      
-		      (let ((spaces (+ col (if (member sym '("or" "and" "cond" "if"))
-					       (+ (length sym) 2)
-					       2))))
-			(if (= cursor-pos (length cur-line))
-			    (begin
-			      (set! cur-line (format #f "~A~NC" cur-line spaces #\space))
-			      (set! cursor-pos (length cur-line)))
-			    (begin
-			      (set! cur-line (format #f "~A~NC~A" 
-						     (substring cur-line 0 cursor-pos)
-						     spaces #\space
-						     (substring cur-line (+ cursor-pos 1))))
-			      (set! cursor-pos (+ cursor-pos spaces))))))))))
+		      (set! col (if (char=? (cur-line i) #\newline) 0 (+ col 1))))
+		    (let* ((sym (do ((i (+ new-red 1) (+ i 1)))
+				    ((not (char-alphabetic? (cur-line i)))
+				     (substring cur-line (+ new-red 1) i))))
+			   (spaces (+ col (if (member sym '("or" "and" "cond" "if"))
+					      (+ (length sym) 2)
+					      2))))
+		      (if (= cursor-pos (length cur-line))
+			  (begin
+			    (set! cur-line (format #f "~A~NC" cur-line spaces #\space))
+			    (set! cursor-pos (length cur-line)))
+			  (begin
+			    (set! cur-line (format #f "~A~NC~A" 
+						   (substring cur-line 0 cursor-pos)
+						   spaces #\space
+						   (substring cur-line (+ cursor-pos 1))))
+			    (set! cursor-pos (+ cursor-pos spaces))))))))))
 	    
 	    
 	    ;; -------- prompt --------
@@ -370,13 +365,13 @@
 	    
 	    
 	    ;; -------- vt100 --------
-	    (define (bold text) (format #f "~C[1m~A~C[0m" #\escape text #\escape)) 
 	    (define (red text) (format #f "~C[31m~A~C[0m" #\escape text #\escape))  ; black=30, green=32, yellow=33, blue=34
 	    
 	    (define* (rgb text (r 0) (g 0) (b 0) all-colors)
-	      (if all-colors
-		  (format #f "~C[38;5;~Dm~A~C[0m" #\escape (+ 16 (* 36 (round (* r 5))) (* 6 (round (* g 5))) (round (* b 5))) text #\escape)
-		  (format #f "~C[~Dm~A~C[0m" #\escape (+ 30 (ash (round b) 2) (ash (round g) 1) (round r)) text #\escape)))
+	      (format #f (if all-colors
+			     (values "~C[38;5;~Dm~A~C[0m" #\escape (+ 16 (* 36 (round (* r 5))) (* 6 (round (* g 5))) (round (* b 5))))
+			     (values "~C[~Dm~A~C[0m" #\escape (+ 30 (ash (round b) 2) (ash (round g) 1) (round r))))
+		      text #\escape))
 	    
 	    (define (move-cursor y x)
 	      (format *stderr* "~C[~D;~DH" #\escape y x))
@@ -433,30 +428,33 @@
 	      (cursor-bounds))
 	    
 	    (define (display-line start end)
-	      
+
+	      (define (bold text) 
+		(format #f "~C[1m~A~C[0m" #\escape text #\escape)) 
+
 	      ;; if a line wraps, it will confuse the redisplay/cursor positioning code. so truncate the display
 	      (let ((line-len (+ (- end start) 1 prompt-length)))
 		(if (>= line-len last-col)
-		    (set! end (- end (- line-len last-col)))))
+		    (set! end (- (+ end line-len) last-col))))
 	      
 	      (if (and red-par-pos
 		       (<= start red-par-pos)
 		       (< red-par-pos end))
 		  (string-append
-		   (if (zero? start)
-		       (format #f "~A" prompt-string)
-		       (format #f "~NC" prompt-length #\space))
-		   (if (= start red-par-pos)
-		       (format #f "~A~A"
-			       (bold (red "(")) 
-			       (substring cur-line (+ start 1) end))
-		       (format #f "~A~A~A"
-			       (substring cur-line start red-par-pos) 
-			       (bold (red "(")) 
-			       (substring cur-line (+ red-par-pos 1) end))))
-		  (if (zero? start)
-		      (format #f "~A~A" prompt-string (substring cur-line 0 end))
-		      (format #f "~NC~A" prompt-length #\space (substring cur-line start end)))))
+		   (format #f (if (zero? start)
+				  (values "~A" prompt-string)
+				  (values "~NC" prompt-length #\space)))
+		   (format #f (if (= start red-par-pos)
+				  (values "~A~A"
+					  (bold (red "(")) 
+					  (substring cur-line (+ start 1) end))
+				  (values "~A~A~A"
+					  (substring cur-line start red-par-pos) 
+					  (bold (red "(")) 
+					  (substring cur-line (+ red-par-pos 1) end)))))
+		  (format #f (if (zero? start)
+				 (values "~A~A" prompt-string (substring cur-line 0 end))
+				 (values "~NC~A" prompt-length #\space (substring cur-line start end))))))
 	    
 	    (define (display-cursor)
 	      (let ((row 0)
@@ -486,13 +484,13 @@
 	    
 	    ;; -------- help/debugging --------
 	    (define (one-line text)
-	      (if (string? text)
+	      (if (not (string? text))
+		  text
 		  (let ((ntext (copy text)))
 		    (do ((i 0 (+ i 1)))
 			((= i (length ntext))
 			 ntext)
-		      (if (char=? (ntext i) #\newline) (set! (ntext i) #\|))))
-		  text))
+		      (if (char=? (ntext i) #\newline) (set! (ntext i) #\|))))))
 	    
 	    (define (help c)
 	      (when (pair? (*repl* 'helpers))
@@ -518,10 +516,10 @@
 		     (lambda (c) 
 		       (format #f "cursor: ~A, ~C, line: ~S"
 			       cursor-pos
-			       (if (> (length cur-line) 0)
+			       (if (zero? (length cur-line))
+				   #\space
 				   (let ((c (cur-line (max 0 (min cursor-pos (- (length cur-line) 1))))))
-				     (if (char=? c #\newline) #\| c))
-				   #\space)
+				     (if (char=? c #\newline) #\| c)))
 			       (one-line cur-line)))
 		     (lambda (c)
 		       (format #f "len: ~D, selection: ~S, previous: ~S" 
@@ -550,27 +548,26 @@
 				     (keymap-functions (char->integer c)))
 				    ((integer? c) 
 				     (keymap-functions c))
-				    ((string? c)
-				     (if (= (length c) 1)
-					 (keymap-functions (char->integer (c 0)))
-					 (if (and (= (length c) 2)
-						  (char=? (c 0) #\escape))
-					     (meta-keymap-functions (char->integer (c 1)))
-					     (lambda (c) #t))))
-				    (else (error 'wrong-type-arg "keymap takes a character or string argument"))))
-			    
+				    ((not (string? c))
+				     (error 'wrong-type-arg "keymap takes a character or string argument"))
+				    ((= (length c) 1) 
+				     (keymap-functions (char->integer (c 0))))
+				    ((and (= (length c) 2)
+					  (char=? (c 0) #\escape))
+				     (meta-keymap-functions (char->integer (c 1))))
+				    (else (lambda (c) #t))))
 			    (lambda (c f)
 			      (cond ((char? c) 
 				     (set! (keymap-functions (char->integer c)) f))
 				    ((integer? c) 
 				     (set! (keymap-functions c) f))
-				    ((string? c)
-				     (if (= (length c) 1)
-					 (set! (keymap-functions (char->integer (c 0))) f)
-					 (if (and (= (length c) 2)
-						  (char=? (c 0) #\escape))
-					     (set! (meta-keymap-functions (char->integer (c 1))) f))))
-				    (else (error 'wrong-type-arg "set! keymap takes a character or string first argument"))))))
+				    ((not (string? c))
+				     (error 'wrong-type-arg "set! keymap takes a character or string first argument"))
+				    ((= (length c) 1)
+				     (set! (keymap-functions (char->integer (c 0))) f))
+				    ((and (= (length c) 2)
+					  (char=? (c 0) #\escape))
+				     (set! (meta-keymap-functions (char->integer (c 1))) f))))))
 	    
 	    (define C-a 1)     ; #\x01 etc
 	    (define C-b 2)
@@ -638,11 +635,11 @@
 	    
 	    (let ((main-keyfunc (lambda (c)
 				  (if (<= chars 1) (save-line))
-				  (if (= cursor-pos (length cur-line))
-				      (set! cur-line (string-append cur-line (string c)))
-				      (if (= cursor-pos 0)
-					  (set! cur-line (string-append (string c) cur-line))
-					  (set! cur-line (string-append 
+				  (set! cur-line (if (= cursor-pos (length cur-line))
+						     (string-append cur-line (string c))
+						     (if (= cursor-pos 0)
+							 (string-append (string c) cur-line)
+							 (string-append 
 							  (substring cur-line 0 cursor-pos) 
 							  (string c) 
 							  (substring cur-line cursor-pos)))))
@@ -770,11 +767,11 @@
 		  (lambda (c)
 		    (when selection
 		      (save-line)
-		      (if (zero? cursor-pos)
-			  (set! cur-line (string-append selection cur-line))
-			  (if (>= cursor-pos (length cur-line))
-			      (set! cur-line (string-append cur-line selection))
-			      (set! cur-line (string-append (substring cur-line 0 cursor-pos)
+		      (set! cur-line (if (zero? cursor-pos)
+					 (string-append selection cur-line)
+					 (if (>= cursor-pos (length cur-line))
+					     (string-append cur-line selection)
+					     (string-append (substring cur-line 0 cursor-pos)
 							    selection
 							    (substring cur-line cursor-pos)))))
 		      (set! cursor-pos (+ cursor-pos (length selection))))))
@@ -789,19 +786,25 @@
 	    ;; -------- add newline
 	    (set! (keymap-functions C-o)
 		  (lambda (c)
-		    (if (= cursor-pos 0)
-			(set! cur-line (string-append (string #\space #\newline) cur-line))
-			(if (>= cursor-pos (length cur-line))
-			    (set! cur-line (string-append cur-line (string #\space #\newline)))
-			    (if (char=? (cur-line (- cursor-pos 1)) #\newline)
-				(set! cur-line (string-append (substring cur-line 0 cursor-pos)
-							      (string #\space #\newline)
-							      (substring cur-line cursor-pos)))
-				(set! cur-line (string-append (substring cur-line 0 cursor-pos)
-							      (if (char=? (cur-line (+ cursor-pos 1)) #\newline)
-								  (string #\space #\newline #\space)
-								  (string #\space #\newline))
-							      (substring cur-line (+ cursor-pos 1)))))))
+		    (set! cur-line (string-append (cond ((= cursor-pos 0) 
+							 (values (string #\space #\newline) 
+								 cur-line))
+
+							((>= cursor-pos (length cur-line)) 
+							 (values cur-line 
+								 (string #\space #\newline)))
+
+							((char=? (cur-line (- cursor-pos 1)) #\newline)
+							 (values (substring cur-line 0 cursor-pos)
+								 (string #\space #\newline)
+								 (substring cur-line cursor-pos)))
+
+							(else
+							 (values (substring cur-line 0 cursor-pos)
+								 (if (char=? (cur-line (+ cursor-pos 1)) #\newline)
+								     (string #\space #\newline #\space)
+								     (string #\space #\newline))
+								 (substring cur-line (+ cursor-pos 1)))))))
 		    (when (= last-row (+ prompt-row cur-row))
 		      (set! prompt-row (- prompt-row 1))
 		      (display-lines))))
@@ -836,26 +839,26 @@
 			  (if (and (positive? start)
 				   (= end start))
 			      (indent start)
-			      (if (= cursor-pos end)
-				  (let ((loc (do ((i (- end 1) (- i 1)))
-						 ((or (< i 0)
-						      (char-whitespace? (cur-line i))
-						      (member (cur-line i) '(#\( #\' #\" #\)) eqv?))
-						  i))))
-				    (if (< loc 0)
-					(set! completion (symbol-completion cur-line))
-					(if (char=? (cur-line loc) #\")
-					    (set! completion (filename-completion (substring cur-line (+ loc 1))))
-					    (set! completion (symbol-completion (substring cur-line (+ loc 1))))))
-				    (when (and completion
-					       (> (length completion) (- end loc 1)))
-				      (save-line)
-				      (if (= end (length cur-line))
-					  (set! cur-line (string-append (substring cur-line 0 (+ loc 1)) completion))
-					  (set! cur-line (string-append (substring cur-line 0 (+ loc 1)) 
-									completion
-									(substring cur-line (+ end 1)))))
-				      (set! cursor-pos (end-of-line cursor-pos))))))))))
+			      (when (= cursor-pos end)
+				(let ((loc (do ((i (- end 1) (- i 1)))
+					       ((or (< i 0)
+						    (char-whitespace? (cur-line i))
+						    (memv (cur-line i) '(#\( #\' #\" #\))))
+						i))))
+				  (set! completion (if (< loc 0)
+						       (symbol-completion cur-line)
+						       (if (char=? (cur-line loc) #\")
+							   (filename-completion (substring cur-line (+ loc 1)))
+							   (symbol-completion (substring cur-line (+ loc 1))))))
+				  (when (and completion
+					     (> (length completion) (- end loc 1)))
+				    (save-line)
+				    (set! cur-line (if (= end (length cur-line))
+						       (string-append (substring cur-line 0 (+ loc 1)) completion)
+						       (string-append (substring cur-line 0 (+ loc 1)) 
+								      completion
+								      (substring cur-line (+ end 1)))))
+				    (set! cursor-pos (end-of-line cursor-pos))))))))))
 	    
 	    ;; -------- evaluation/multiline
 	    (set! (keymap-functions Enter) 
@@ -889,7 +892,7 @@
 				 (if (or (= chars 1)
 					 (not (= input-fd terminal-fd)))
 				     (display-lines))
-				 (set! (history) (copy cur-line))
+				 (set! (history) cur-line)
 				 (set! m-p-pos 0)
 				 
 				 (with-repl-let
@@ -902,6 +905,11 @@
 					    (set! unbound-case #f)
 					    (begin
 					      (format *stderr* "~S~%" val)
+					      ;; this set! of '** has one odd consequence: if val is a lambda expression
+					      ;;   find_closure in s7 will fallback on the current environment trying to
+					      ;;   find an associated name, and the only thing it finds is '**!  So,
+					      ;;   when we type **, we get back **, which seems perverse.  I suppose
+					      ;;   we could trap the string above, see "**" and change to ~W or something.
 					      (set! ((*repl* 'top-level-let) '**) val))))))))
 			       
 			       (lambda (type info)
@@ -910,8 +918,12 @@
 				 (return))))
 			   
 			   (lambda (type info)
-			     (format *stderr* "~A: " (red "error"))
-			     (apply format *stderr* info)
+			     (format *stderr* "~A:" (red "error"))
+			     (if (and (pair? info)
+				      (string? (car info)))
+				 (format *stderr* " ~A" (apply format #f info))
+				 (if (not (null? info))
+				     (format *stderr* " ~A" info)))
 			     (newline *stderr*)))
 			 
 			 (push-line (copy cur-line))
@@ -944,11 +956,11 @@
 		  (set! cur-row newlines)))
 
 		(define (get-previous-line c)         ; get earlier line indexed by numeric arg
-		  (let ((len (length histtop)))
-		    (let ((pos (or (string->number cur-line) len)))
-		      (set! pos (min len (max pos 1)))
-		      (set! cur-line (histtop (- pos 1)))
-		      (fixup-new-line))))
+		  (let* ((len (length histtop))
+			 (pos (or (string->number cur-line) len)))
+		    (set! pos (min len (max pos 1)))
+		    (set! cur-line (copy (histtop (- pos 1))))
+		    (fixup-new-line)))
 
 		(define (move-forward-in-history c)
 		  (set! m-p-pos (min 0 (+ m-p-pos 1)))
@@ -1030,7 +1042,7 @@
 		    (tty #t))
 		
 		;; we're in libc here, so exit is libc's exit!
-		(define (tty-reset no)
+		(define (tty-reset)
 		  (if tty (tcsetattr terminal-fd TCSAFLUSH saved))
 		  (if (not (equal? input-fd terminal-fd)) (close input-fd))
 		  (#_exit))
@@ -1039,7 +1051,7 @@
 		  :exit (let ((documentation "(exit) resets the repl tty and exits the repl"))
 			  (lambda ()
 			    (newline *stderr*)
-			    (tty-reset 0))))
+			    (tty-reset))))
 		
 		;; check for dumb terminal
 		(if (or (zero? (isatty terminal-fd))        ; not a terminal -- input from pipe probably
@@ -1068,109 +1080,107 @@
 				   (format *stderr* "> ")))))))))
 		
 		;; not a pipe or a dumb terminal -- hopefully all others accept vt100 codes
-		(let ((buf (termios.make))
-		      (read-size 128))
-		  
-		  (set! next-char                                     ; this indirection is needed if user pastes the selection into the repl
-			(let* ((c (make-string read-size #\null)) 
-			       (cc (string->c-pointer c))
-			       (ctr 0))
-			  (lambda ()
-			    (call-with-exit
-			     (lambda (return)
-			       (when (>= ctr chars)
-				 (set! ctr 0)
-				 (set! chars (read input-fd cc read-size))
-				 (if (= chars 0)
-				     (tty-reset terminal-fd))
-				 
-				 (when (= chars read-size)
-				   ;; concatenate buffers until we get the entire selection
-				   (let ((str (substring c 0 read-size)))
-				     (let reading ((num (read input-fd cc read-size)))
-				       (set! str (string-append str (substring c 0 num)))
-				       (set! chars (+ chars num))
-				       (if (= num read-size)
-					   (reading (read input-fd cc read-size))))
-				     
-				     ;; look for simple cut/paste -- no control chars etc
-				     (when (= input-fd terminal-fd)
-				       (let ((bcksp (integer->char 127))
-					     (ok-chars (list #\newline #\linefeed #\return #\tab)))
-					 (do ((i 0 (+ i 1)))
-					     ((or (= i chars)
-						  (char>=? (str i) bcksp)
-						  (and (char<? (str i) #\space)
-						       (not (member (str i) ok-chars eq?))))
-					      
-					      (when (= i chars)
-						(let ((search-chars (string #\tab #\return #\newline))
-						      (old-pos 0)
-						      (start 0)
-						      (max-cols 0))
-						  (do ((pos (char-position search-chars str 0) (char-position search-chars str (+ pos 1))))
-						      ((not pos))
-						    (set! cur-line (string-append cur-line 
-										  (substring str old-pos pos)
-										  (if (char=? (str pos) #\tab) 
-										      tab-as-space 
-										      (string #\space #\newline))))
-						    (set! old-pos (+ pos 1))
-						    (unless (char=? (str pos) #\tab)
-						      (set! max-cols (max max-cols (- pos start)))
-						      (set! start pos)))
-						  (if (< (+ old-pos 1) (length str))
-						      (set! cur-line (string-append cur-line (substring str (+ old-pos 1)))))
-						  
-						  ;; if the line is too long, the cursor gets confused, so try to reformat over-long strings
-						  ;; this still messes up sometimes
-
-						  (when (> max-cols (- last-col prompt-length))
-						    (let ((old-len ((funclet pretty-print) '*pretty-print-length*)))
-						      (set! ((funclet pretty-print) '*pretty-print-length*) (- last-col prompt-length 2))
-						      (set! cur-line (with-output-to-string
-								       (lambda ()
-									 (pretty-print (with-input-from-string cur-line #_read)))))
-						      (set! ((funclet pretty-print) '*pretty-print-length*) old-len)))
-						  
+		(let ((buf (termios.make)))
+		  (let ((read-size 128))
+		    (set! next-char                                     ; this indirection is needed if user pastes the selection into the repl
+			  (let* ((c (make-string read-size #\null)) 
+				 (cc (string->c-pointer c))
+				 (ctr 0))
+			    (lambda ()
+			      (call-with-exit
+			       (lambda (return)
+				 (when (>= ctr chars)
+				   (set! ctr 0)
+				   (set! chars (read input-fd cc read-size))
+				   (if (= chars 0)
+				       (tty-reset))
+				   
+				   (when (= chars read-size)
+				     ;; concatenate buffers until we get the entire selection
+				     (let ((str (substring c 0 read-size)))
+				       (let reading ((num (read input-fd cc read-size)))
+					 (set! str (string-append str (substring c 0 num)))
+					 (set! chars (+ chars num))
+					 (if (= num read-size)
+					     (reading (read input-fd cc read-size))))
+				       
+				       ;; look for simple cut/paste -- no control chars etc
+				       (when (= input-fd terminal-fd)
+					 (let ((bcksp #\delete)
+					       (ok-chars (list #\newline #\linefeed #\return #\tab)))
+					   (do ((i 0 (+ i 1)))
+					       ((or (= i chars)
+						    (char>=? (str i) bcksp)
+						    (and (char<? (str i) #\space)
+							 (not (memv (str i) ok-chars))))
+						
+						(when (= i chars)
+						  (let ((search-chars (string #\tab #\return #\newline))
+							(old-pos 0)
+							(start 0)
+							(max-cols 0))
+						    (do ((pos (char-position search-chars str 0) (char-position search-chars str (+ pos 1))))
+							((not pos))
+						      (set! cur-line (string-append cur-line 
+										    (substring str old-pos pos)
+										    (if (char=? (str pos) #\tab) 
+											tab-as-space 
+											(string #\space #\newline))))
+						      (set! old-pos (+ pos 1))
+						      (unless (char=? (str pos) #\tab)
+							(set! max-cols (max max-cols (- pos start)))
+							(set! start pos)))
+						    (if (< (+ old-pos 1) (length str))
+							(set! cur-line (string-append cur-line (substring str (+ old-pos 1)))))
+						    
+						    ;; if the line is too long, the cursor gets confused, so try to reformat over-long strings
+						    ;; this still messes up sometimes
+						    
+						    (when (> max-cols (- last-col prompt-length))
+						      (let ((old-len ((funclet pretty-print) '*pretty-print-length*)))
+							(set! ((funclet pretty-print) '*pretty-print-length*) (- last-col prompt-length 2))
+							(set! cur-line (with-output-to-string
+									 (lambda ()
+									   (pretty-print (with-input-from-string cur-line #_read)))))
+							(set! ((funclet pretty-print) '*pretty-print-length*) old-len))))
+						    
 						  (set! cursor-pos (length cur-line))
 						  (set! chars 0)
 						  (set! ctr 1)
 						  (display-lines)
-						  (return #\newline)))))))
-				     
-				     (set! c str)
-				     (set! cc (string->c-pointer c))
-				     ;; avoid time-consuming redisplays.  We need to use a recursive call on next-char here
-				     ;;   since we might have multi-char commands (embedded #\escape -> meta, etc)
-				     ;; actually, the time is not the repl's fault -- xterm seems to be waiting
-				     ;;   for the window manager or someone to poke it -- if I move the mouse,
-				     ;;   I get immediate output.  I also get immediate output in any case in OSX.
-				     ;;   valgrind and ps say we're not computing, we're just sitting there.
-				     (catch #t
-				       (lambda ()
-					 (do ((ch (next-char) (next-char)))
-					     ((= ctr (- chars 1)) 
-					      (set! chars 0)
-					      (display-lines)
-					      (return ch))
-					   ((keymap-functions (char->integer ch)) ch)))
+						  (return #\newline))))))
 				       
-				       (lambda (type info)
-					 (set! chars 0)
-					 (move-cursor prompt-row prompt-col)
-					 (format *stderr* "internal error: ")
-					 (apply format *stderr* info)
-					 (newline *stderr*)
-					 (format *stderr* "line ~A: ~A~%" ((owlet) 'error-line) ((owlet) 'error-code))
-					 (set! chars 0)
-					 (set! ctr 0)
-					 (new-prompt)
-					 (return #\null))))))
-			       
-			       (let ((result (c ctr)))
-				 (set! ctr (+ ctr 1))
-				 result))))))
+				       (set! c str)
+				       (set! cc (string->c-pointer c))
+				       ;; avoid time-consuming redisplays.  We need to use a recursive call on next-char here
+				       ;;   since we might have multi-char commands (embedded #\escape -> meta, etc)
+				       ;; actually, the time is not the repl's fault -- xterm seems to be waiting
+				       ;;   for the window manager or someone to poke it -- if I move the mouse,
+				       ;;   I get immediate output.  I also get immediate output in any case in OSX.
+				       ;;   valgrind and ps say we're not computing, we're just sitting there.
+				       (catch #t
+					 (lambda ()
+					   (do ((ch (next-char) (next-char)))
+					       ((= ctr (- chars 1)) 
+						(set! chars 0)
+						(display-lines)
+						(return ch))
+					     ((keymap-functions (char->integer ch)) ch)))
+					 
+					 (lambda (type info)
+					   (set! chars 0)
+					   (move-cursor prompt-row prompt-col)
+					   (format *stderr* "internal error: ")
+					   (apply format *stderr* info)
+					   (format *stderr* "~%line ~A: ~A~%" ((owlet) 'error-line) ((owlet) 'error-code))
+					   (set! chars 0)
+					   (set! ctr 0)
+					   (new-prompt)
+					   (return #\null))))))
+				 
+				 (let ((result (c ctr)))
+				   (set! ctr (+ ctr 1))
+				   result)))))))
 		  
 		  (set! input-fd (if (not file) 
 				     terminal-fd
@@ -1184,7 +1194,7 @@
 		  (termios.set_c_cc buf VMIN 1)
 		  (termios.set_c_cc buf VTIME 0)
 		  (when (negative? (tcsetattr terminal-fd TCSAFLUSH buf))
-		    (tty-reset terminal-fd))
+		    (tty-reset))
 		  
 		  
 		  ;; -------- the repl --------
@@ -1319,27 +1329,27 @@
 (define* (apropos name (e (*repl* 'top-level-let)))
   
   (define (levenshtein s1 s2)
-    (let ((l1 (length s1))
-	  (l2 (length s2)))
-      (cond ((zero? l1) l2)
-	    ((zero? l2) l1)
-	    (else (let ((distance (make-vector (list (+ l2 1) (+ l1 1)) 0)))
+    (let ((L1 (length s1))
+	  (L2 (length s2)))
+      (cond ((zero? L1) L2)
+	    ((zero? L2) L1)
+	    (else (let ((distance (make-vector (list (+ L2 1) (+ L1 1)) 0)))
 		    (do ((i 0 (+ i 1)))
-			((> i l1))
+			((> i L1))
 		      (set! (distance 0 i) i))
 		    (do ((i 0 (+ i 1)))
-			((> i l2))
+			((> i L2))
 		      (set! (distance i 0) i))
 		    (do ((i 1 (+ i 1)))
-			((> i l2))
+			((> i L2))
 		      (do ((j 1 (+ j 1)))
-			  ((> j l1))
+			  ((> j L1))
 			(let ((c1 (+ (distance i (- j 1)) 1))
 			      (c2 (+ (distance (- i 1) j) 1))
 			      (c3 (+ (distance (- i 1) (- j 1)) 
 				     (if (char=? (s2 (- i 1)) (s1 (- j 1))) 0 1))))
 			  (set! (distance i j) (min c1 c2 c3)))))
-		    (distance l2 l1))))))
+		    (distance L2 L1))))))
 
   (define* (make-full-let-iterator lt (stop (rootlet))) ; walk the entire let chain
     (if (eq? stop lt)
@@ -1350,12 +1360,11 @@
 		    (lambda ()
 		      (let ((result (iter)))
 			(if (and (eof-object? result)
-				 (iterator-at-end? iter))
-			    (if (eq? stop (iterator-sequence iter))
-				result
-				(begin 
-				  (set! iter (make-iterator (outlet (iterator-sequence iter))))
-				  (iterloop)))
+				 (iterator-at-end? iter)
+				 (not (eq? stop (iterator-sequence iter))))
+			    (begin 
+			      (set! iter (make-iterator (outlet (iterator-sequence iter))))
+			      (iterloop))
 			    result))))))
 	    (make-iterator iterloop))))
   
@@ -1378,7 +1387,8 @@
 			 (set! strs (cons (cons binding distance) strs))))))))
        (make-full-let-iterator ap-env))
 
-      (if (pair? strs)
+      (if (not (pair? strs))
+	  'no-match
 	  (begin
 	    (for-each (lambda (b)
 			(format *stderr*
@@ -1387,16 +1397,14 @@
 					"~C[38;5;208m~A~C[0m: ~S~%"))                              ; orange for less likely choices
 				#\escape (caar b) #\escape
 				(if (procedure? (cdar b))
-				    (let ((doc (procedure-documentation (cdar b))))
-				      (if (and (string? doc)
-					       (positive? (length doc)))
+				    (let ((doc (procedure-documentation (cdar b)))) ; returns "" if no doc
+				      (if (positive? (length doc))
 					  doc
 					  'procedure))
 				    (cdar b))))
 		      (sort! strs (lambda (a b)
 				    (string<? (symbol->string (caar a)) (symbol->string (caar b))))))
-	    '----)
-	  'no-match))))
+	    '----)))))
 
 
 ;;; --------------------------------------------------------------------------------
