@@ -149,18 +149,16 @@
 		       (sr0 (make-sampler (floor s0)))
 		       (sr1 (make-sampler (floor s1)))
 		       (ampsum (make-one-pole 1.0 -1.0))
-		       (diffsum (make-one-pole 1.0 -1.0))
-		       (samp0 0.0))
-		  (do ((i 0 (+ i 1)))
+		       (diffsum (make-one-pole 1.0 -1.0)))
+		  (do ((samp0 0.0)
+		       (i 0 (+ i 1)))
 		      ((= i len))
 		    (set! samp0 (next-sample sr0))
 		    (one-pole ampsum (abs samp0))
 		    (one-pole diffsum (abs (- (next-sample sr1) samp0))))
 		  (set! diffsum (one-pole diffsum 0.0))
 		  (set! ampsum (one-pole ampsum 0.0))
-		  (if (= diffsum 0.0)
-		      (set! current-min 0.0)
-		      (set! current-min (/ diffsum ampsum))))
+		  (set! current-min (if (= diffsum 0.0) 0.0 (/ diffsum ampsum))))
 		(set! min-samps (round (* 0.5 current-min)))
 		(let ((top (min (- crosses 1) current-mark (+ i zeros-checked))))
 		  (do ((k (+ i 1) (+ k 1)))
@@ -172,18 +170,16 @@
 			     (sr0 (make-sampler (floor s0)))
 			     (sr1 (make-sampler (floor s1)))
 			     (ampsum (make-one-pole 1.0 -1.0))
-			     (diffsum (make-one-pole 1.0 -1.0))
-			     (samp0 0.0))
-			(do ((i 0 (+ i 1)))
+			     (diffsum (make-one-pole 1.0 -1.0)))
+			(do ((samp0 0.0)
+			     (i 0 (+ i 1)))
 			    ((= i len))
 			  (set! samp0 (next-sample sr0))
 			  (one-pole ampsum (abs samp0))
 			  (one-pole diffsum (abs (- (next-sample sr1) samp0))))
 			(set! diffsum (one-pole diffsum 0.0))
 			(set! ampsum (one-pole ampsum 0.0))
-			(if (= diffsum 0.0)
-			    (set! wgt 0.0)
-			    (set! wgt (/ diffsum ampsum))))
+			(set! wgt (if (= diffsum 0.0) 0.0 (/ diffsum ampsum))))
 		      (if (< wgt min-samps)
 			  (begin
 			    (set! min-samps (floor wgt))
@@ -241,38 +237,38 @@
 		    (beg (floor (cross-samples best-mark)))
 		    (next-beg (floor (cross-samples (floor (cross-marks best-mark)))))
 		    (len (floor (cross-periods best-mark))))
-	       (if (> len 0)
-		   (if adding
-		       (let ((new-samps
-			      (env-add beg next-beg len)))
-			 (if show-details
-			     (add-named-mark beg (format #f "~D:~D" i (floor (/ len extension)))))
-			 (insert-samples beg len new-samps)
-			 (if (> mult 1)
-			     (do ((k 1 (+ k 1)))
-				 ((= k mult))
-			       (insert-samples (+ beg (* k len)) len new-samps)))
-			 (set! changed-len (+ changed-len (* mult len)))
+	       (when (> len 0)
+		 (if adding
+		     (let ((new-samps
+			    (env-add beg next-beg len)))
+		       (if show-details
+			   (add-named-mark beg (format #f "~D:~D" i (floor (/ len extension)))))
+		       (insert-samples beg len new-samps)
+		       (if (> mult 1)
+			   (do ((k 1 (+ k 1)))
+			       ((= k mult))
+			     (insert-samples (+ beg (* k len)) len new-samps)))
+		       (set! changed-len (+ changed-len (* mult len)))
+		       (do ((j 0 (+ 1 j)))
+			   ((= j weights))
+			 (let ((curbeg (floor (cross-samples j))))
+			   (if (> curbeg beg)
+			       (set! (cross-samples j) (+ curbeg len))))))
+		     (begin
+		       (if (>= beg (framples))
+			   (snd-print (format #f "trouble at ~D: ~D of ~D~%" i beg (framples))))
+		       (if show-details
+			   (add-named-mark (- beg 1) (format #f "~D:~D" i (floor (/ len extension)))))
+		       (delete-samples beg len)
+		       (set! changed-len (+ changed-len len))
+		       (let ((end (+ beg len)))
 			 (do ((j 0 (+ 1 j)))
 			     ((= j weights))
 			   (let ((curbeg (floor (cross-samples j))))
 			     (if (> curbeg beg)
-				 (set! (cross-samples j) (+ curbeg len))))))
-		       (begin
-			 (if (>= beg (framples))
-			     (snd-print (format #f "trouble at ~D: ~D of ~D~%" i beg (framples))))
-			 (if show-details
-			     (add-named-mark (- beg 1) (format #f "~D:~D" i (floor (/ len extension)))))
-			 (delete-samples beg len)
-			 (set! changed-len (+ changed-len len))
-			 (let ((end (+ beg len)))
-			   (do ((j 0 (+ 1 j)))
-			       ((= j weights))
-			     (let ((curbeg (floor (cross-samples j))))
-			       (if (> curbeg beg)
-				   (if (< curbeg end)
-				       (set! (cross-periods j) 0)
-				       (set! (cross-samples j) (- curbeg len))))))))))))
+				 (if (< curbeg end)
+				     (set! (cross-periods j) 0)
+				     (set! (cross-samples j) (- curbeg len))))))))))))
 	   (if show-details
 	       (snd-print (format #f "wanted: ~D, got ~D~%" (floor samps) (floor changed-len)))))
 	 ))
