@@ -32,15 +32,13 @@
 	 (durs (map car data))
 	 (dur (apply + durs))
 	 (begs (let ((bg beg))
-		 (append (list beg)
-			 (map (lambda (x)
-				(set! bg (+ bg x)))
-			      durs))))
+		 (cons beg
+		       (map (lambda (x)
+			      (set! bg (+ bg x)))
+			    durs))))
 	 (beg-samps (map seconds->samples begs)))
 
-    (let ((change-times (let* ((len (length beg-samps))
-			       (nbegs (append beg-samps (list (beg-samps (- len 1))))))
-			  (apply vector nbegs)))
+    (let ((change-times (apply vector (append beg-samps (list (beg-samps (- (length beg-samps) 1))))))
 	  
 	  (shps (map cadr data))
 	  (glts (map caddr data))
@@ -256,8 +254,8 @@
 		      (set! temp (/ one-over-two-pi b-a))
 		      (set! temp1 (- 1.0 ca2))
 		      (set! (sines 1) (* (+ ca2 (* (- sa2 (sin b2)) temp)) temp1 one-over-two-pi))
-		      (set! (cosines 1) (* (+ (- sa2) (* (- ca2 (cos b2)) temp)) temp1 one-over-two-pi))))
-		(set! (sines 1) (+ (sines 1) (* (+ 0.75 (- ca2) (* (cos (* 2 a2)) 0.25)) one-over-two-pi)))
+		      (set! (cosines 1) (* (- (* (- ca2 (cos b2)) temp) sa2) temp1 one-over-two-pi))))
+		(set! (sines 1) (+ (sines 1) (* (- (+ 0.75 (* (cos (* 2 a2)) 0.25)) ca2) one-over-two-pi)))
 		(set! (cosines 1) (+ (cosines 1) (- (* (- sa2 (* (sin (* 2 a2)) 0.25)) one-over-two-pi) (* a 0.5))))
 		(do ((k 2 (+ k 1))
 		     (ka2 (* 2 a2) (+ ka2 a2))
@@ -268,7 +266,7 @@
 		      (begin
 			(set! temp (/ one-over-two-pi (* b-a k)))
 			(set! (sines k) (* (+ (cos ka2) (* (- (sin ka2) (sin (* k b2))) temp)) (/ temp1 k)))
-			(set! (cosines k) (* (+ (- (sin ka2)) (* (- (cos ka2) (cos (* k b2))) temp)) (/ temp1 k)))))
+			(set! (cosines k) (* (- (* (- (cos ka2) (cos (* k b2))) temp) (sin ka2)) (/ temp1 k)))))
 		  (set! (sines k) (+ (sines k) 
 				     (/ (- 1.0 (cos ka2)) k)
 				     (/ (* (- (cos ka1) 1.0) 0.5) (- k 1))
@@ -307,10 +305,7 @@
 			  (if (> (abs (- (shape-data j) (shape-data k))) .001)
 			      (set! new-tract #t)))
 			(set! last-sfd new-sfd)))
-		  (if (= last-gfd -1)
-		      (set! last-gfd 0)
-		      (let ((new-gfd (+ last-gfd 2)))
-			(set! last-gfd new-gfd)))
+		  (set! last-gfd (if (= last-gfd -1) 0 (+ last-gfd 2)))
 		  (set! next-offset (floor (change-times (+ offset 1))))
 		  (set! delta (/ 1.0 (- next-offset i)))))
 	    
@@ -328,70 +323,66 @@
 		    (if (> (abs (- (target-radii j) (radii j))) 0.001)
 			(set! change-radii #t)))))
 	    
-	    (if (or first-tract change-radii)
-		(begin
-		  (if (not new-tract)
-		      (begin
-			(float-vector-multiply! radii radii-poles)
-			(copy target-radii target-temp)
-			(float-vector-multiply! target-temp radii-pole-gains)
-			(float-vector-add! radii target-temp)
-			;; (do ((j 0 (+ j 1))) ((= j tractlength+8))
-			;;   (float-vector-set! radii j (+ (* (float-vector-ref radii j) (float-vector-ref radii-poles j))
-			;; 		                   (* (float-vector-ref target-radii j) (float-vector-ref radii-pole-gains j)))))
-			))
-		  ;; set tract shape
-		  (let ((tj 1.0)
-			(tk 0.0))
-		    (do ((k 0 (+ k 1))
-			 (j 1 (+ j 1)))
-			((= j tractlength))
-		      (set! tk tj)
-		      (if (zero? (float-vector-ref radii j))
-			  (set! tj 1e-10)
-			  (set! tj (* (float-vector-ref radii k) (float-vector-ref radii k))))
-		      (float-vector-set! coeffs j (/ (- tk tj) (+ tk tj)))))
-
-		  (set! glot-refl-gain (radii tractlength-1))
-		  (set! lip-refl-gain (radii tractlength))
-		  (set! noise-pos (floor (radii tractlength+1)))
-		  (set! noise-gain (radii (+ tractlength 2)))
-
-		  (let ((temp1 (radii (+ tractlength 3)))
-			(r (radii (+ tractlength 4)))
-			(t2 (radii (+ tractlength 5)))
-			(r2 (radii (+ tractlength 6))))
-		    (let (;; fricative noise generator (set noise angle and radius)
-			  (noise-angle (hz->radians temp1))
-			  (noise-angle2 (hz->radians t2))
-			  (noise-radius r)
-			  (noise-radius2 r2))
-		      (let ((noise-a (* -2.0 (cos (/ noise-angle formant-shift)) noise-radius))
-			    (noise-b (* noise-radius noise-radius))
-			    (noise-a2 (* -2.0 (cos (/ noise-angle2 formant-shift)) noise-radius2))
-			    (noise-b2 (* noise-radius2 noise-radius2)))
-			(set! (noisev 0) (+ noise-a noise-a2))
-			(set! (noisev 1) (+ noise-b noise-b2 (* noise-a noise-a2)))
-			(set! (noisev 2) (+ (* noise-a2 noise-b) (* noise-b2 noise-a)))
-			(set! (noisev 3) (* noise-b2 noise-b)))))
-		  
-		  (set! lip-radius (radii tractlength-2))
-		  (set! velum-pos (radii (+ tractlength 7)))
-		  (let ((leftradius (radii (- noseposition 2)))
-			(velumradius velum-pos)
-			(rightradius (radii (- noseposition 1))))
-		    (let ((temp1 0.0) 
-			  (temp 0.0))
-		      ;; nasal tract (set nasal shape)
-		      (set! temp (- rightradius velumradius))
-		      (if (< temp 0.0) (set! temp 0.0))
-		      (set! alpha1 (* leftradius leftradius))
-		      (set! alpha2 (* temp temp))
-		      (set! alpha3 (* velumradius velumradius))
-		      (set! temp1 (/ 2.0 (+ alpha1 alpha2 alpha3)))
-		      (set! alpha1 (* alpha1 temp1))
-		      (set! alpha2 (* alpha2 temp1))
-		      (set! alpha3 (* alpha3 temp1))))))
+	    (when (or first-tract change-radii)
+	      (if (not new-tract)
+		  (begin
+		    (float-vector-multiply! radii radii-poles)
+		    (copy target-radii target-temp)
+		    (float-vector-multiply! target-temp radii-pole-gains)
+		    (float-vector-add! radii target-temp)
+		    ;; (do ((j 0 (+ j 1))) ((= j tractlength+8))
+		    ;;   (float-vector-set! radii j (+ (* (float-vector-ref radii j) (float-vector-ref radii-poles j))
+		    ;; 		                   (* (float-vector-ref target-radii j) (float-vector-ref radii-pole-gains j)))))
+		    ))
+	      ;; set tract shape
+	      (do ((tj 1.0)
+		   (tk 0.0)
+		   (k 0 (+ k 1))
+		   (j 1 (+ j 1)))
+		  ((= j tractlength))
+		(set! tk tj)
+		(set! tj (if (zero? (float-vector-ref radii j))
+			     1e-10
+			     (* (float-vector-ref radii k) (float-vector-ref radii k))))
+		(float-vector-set! coeffs j (/ (- tk tj) (+ tk tj))))
+	      
+	      (set! glot-refl-gain (radii tractlength-1))
+	      (set! lip-refl-gain (radii tractlength))
+	      (set! noise-pos (floor (radii tractlength+1)))
+	      (set! noise-gain (radii (+ tractlength 2)))
+	      
+	      (let ((temp1 (radii (+ tractlength 3)))
+		    (r (radii (+ tractlength 4)))
+		    (t2 (radii (+ tractlength 5)))
+		    (r2 (radii (+ tractlength 6))))
+		(let (;; fricative noise generator (set noise angle and radius)
+		      (noise-angle (hz->radians temp1))
+		      (noise-angle2 (hz->radians t2))
+		      (noise-radius r)
+		      (noise-radius2 r2))
+		  (let ((noise-a (* -2.0 (cos (/ noise-angle formant-shift)) noise-radius))
+			(noise-b (* noise-radius noise-radius))
+			(noise-a2 (* -2.0 (cos (/ noise-angle2 formant-shift)) noise-radius2))
+			(noise-b2 (* noise-radius2 noise-radius2)))
+		    (set! (noisev 0) (+ noise-a noise-a2))
+		    (set! (noisev 1) (+ noise-b noise-b2 (* noise-a noise-a2)))
+		    (set! (noisev 2) (+ (* noise-a2 noise-b) (* noise-b2 noise-a)))
+		    (set! (noisev 3) (* noise-b2 noise-b)))))
+	      
+	      (set! lip-radius (radii tractlength-2))
+	      (set! velum-pos (radii (+ tractlength 7)))
+	      (let ((leftradius (radii (- noseposition 2)))
+		    (velumradius velum-pos)
+		    (rightradius (radii (- noseposition 1))))
+		  (let ((temp (max (- rightradius velumradius) 0.0)))
+		    ;; nasal tract (set nasal shape)
+		    (set! alpha1 (* leftradius leftradius))
+		    (set! alpha2 (* temp temp)))
+		  (set! alpha3 (* velumradius velumradius)))
+	      (let ((temp1 (/ 2.0 (+ alpha1 alpha2 alpha3))))
+		(set! alpha1 (* alpha1 temp1))
+		(set! alpha2 (* alpha2 temp1))
+		(set! alpha3 (* alpha3 temp1))))
 	    
 	    (if new-tract
 		(begin
@@ -414,9 +405,9 @@
 		  (set! table-location (+ table-location table-increment))
 		  (if (>= table-location table-size)
 		      (set! table-location (- table-location table-size)))
-		  (let ((int-loc (floor table-location)))
-		    (let ((table1 (glot-table int-loc)))
-		      (set! glotsamp (+ glotsamp (* (env glot-env) (+ table1 (* s-glot-mix (- (glot-table2 int-loc) table1))))))))))
+		  (let* ((int-loc (floor table-location))
+			 (table1 (glot-table int-loc)))
+		    (set! glotsamp (+ glotsamp (* (env glot-env) (+ table1 (* s-glot-mix (- (glot-table2 int-loc) table1)))))))))
 	    
 	    ;; next tract tick
 	    (let ((j 0)
@@ -450,9 +441,10 @@
 		      (set! nose-last-minus-refl (- nose-reftemp plussamp))
 		      (set! nose-last-plus-refl (- nose-reftemp minussamp)))
 		    (begin
-		      (if (not (= velum-pos 0.0))
-			  (set! time-nose-closed 0)
-			  (set! time-nose-closed (+ time-nose-closed 1))) ; added 1 bil 17-Apr-11 but didn't test it
+		      (set! time-nose-closed 
+			    (if (= velum-pos 0.0)
+				(+ time-nose-closed 1) ; added 1 bil 17-Apr-11 but didn't test it
+				0))
 		      ;; nasal tick
 		      (let ((nose-reftemp (+ (* alpha1 plussamp) (* alpha2 minussamp) (* alpha3 (nose2 1)))))
 			(let (;(nose-t1 0.0)
@@ -462,9 +454,8 @@
 			  (set! nose-last-plus-refl (- nose-reftemp minussamp))
 			  (set! nose-reftemp (* (nose-coeffs 1) (- plus-in (nose2 2))))
 			  (set! (nose2 1) (+ (nose2 2) nose-reftemp))
-			  (set! nose-temp (+ plus-in nose-reftemp))
 			  
-			  (set! nose-temp (singer-nose-filter noselength-1 nose-temp nose1 nose2 nose-coeffs))
+			  (set! nose-temp (singer-nose-filter noselength-1 (+ plus-in nose-reftemp) nose1 nose2 nose-coeffs))
 #|
 			  (do ((j 2 (+ j 1))
 			       (k 1 (+ k 1)))
@@ -484,12 +475,10 @@
 			  (set! nose-filt (nose1 noselength-1))
 			  (set! nose-last-output (* (+ nose-filt nose-filt1) 0.5))))))
 		(set! (dline2 j) nose-last-minus-refl))
-		
 	      (set! (dline1 (- j 1)) temp)
-	      (set! temp nose-last-plus-refl)
 	      
 	      ;; j always starts at 4, goes to 8 so this loop can be unrolled, but doing so doesn't make a big difference
-	      (set! temp (singer-filter noseposition tractlength-1 temp dline1 dline2 coeffs))
+	      (set! temp (singer-filter noseposition tractlength-1 nose-last-plus-refl dline1 dline2 coeffs))
 #|
 	      (let ((x 0.0))
 		(do ((j (+ noseposition 1) (+ j 1))
