@@ -1,7 +1,8 @@
 #ifndef SND_G1_H
 #define SND_G1_H
 
-#define SOUND_ENV_EDITOR(Sp) ((env_editor *)(sp->sgx->flt))
+void recolor_everything(widget_t w, gpointer color);
+
 
 /* -------- snd-ghelp.c -------- */
 
@@ -10,27 +11,26 @@ GtkWidget *snd_help_with_xrefs(const char *subject, const char *helpstr, with_wo
 int help_text_width(const char *txt, int start, int end);
 void snd_help_append(const char *text);
 void snd_help_back_to_top(void);
-bool help_dialog_is_active(void);
+
 
 
 /* -------- snd-gdraw.c -------- */
 
-void draw_line(axis_context *ax, int x0, int y0, int x1, int y1);
-void draw_lines(axis_context *ax, point_t *points, int num);
-void draw_points(axis_context *ax, point_t *points, int num, int size);
-void fill_rectangle(axis_context *ax, int x0, int y0, int width, int height);
-void erase_rectangle(chan_info *cp, axis_context *ax, int x0, int y0, int width, int height);
-void fill_polygon(axis_context *ax, int points, ...);
-void draw_polygon(axis_context *ax, int points, ...);
-void fill_polygons(axis_context *ax, point_t *points, int num, int y0);
-void fill_two_sided_polygons(axis_context *ax, point_t *points, point_t *points1, int num);
-void fill_polygon_from_array(axis_context *ax, point_t *points, int npoints);
-void draw_picture(axis_context *ax, picture_t *src, gint xsrc, gint ysrc, gint xdest, gint ydest, gint width, gint height);
-void draw_string(axis_context *ax, int x0, int y0, const char *str, int len);
-void draw_rotated_axis_label(chan_info *cp, axis_context *ax, const char *text, gint x0, gint y0);
-void draw_dot(axis_context *ax, int x, int y, int size);
-void draw_colored_lines(chan_info *cp, axis_context *ax, point_t *points, int num, int *colors, int axis_y0, color_t default_color);
-void setup_axis_context(chan_info *cp, axis_context *ax);
+void draw_line(graphics_context *ax, int x0, int y0, int x1, int y1);
+void draw_lines(graphics_context *ax, point_t *points, int num);
+void draw_points(graphics_context *ax, point_t *points, int num, int size);
+void fill_rectangle(graphics_context *ax, int x0, int y0, int width, int height);
+void erase_rectangle(chan_info *cp, graphics_context *ax, int x0, int y0, int width, int height);
+void fill_polygon(graphics_context *ax, int points, ...);
+void fill_polygons(graphics_context *ax, point_t *points, int num, int y0);
+void fill_two_sided_polygons(graphics_context *ax, point_t *points, point_t *points1, int num);
+void fill_polygon_from_array(graphics_context *ax, point_t *points, int npoints);
+void draw_picture(graphics_context *ax, picture_t *src, gint xsrc, gint ysrc, gint xdest, gint ydest, gint width, gint height);
+void draw_string(graphics_context *ax, int x0, int y0, const char *str, int len);
+void draw_rotated_axis_label(chan_info *cp, graphics_context *ax, const char *text, gint x0, gint y0);
+void draw_dot(graphics_context *ax, int x, int y, int size);
+void draw_colored_lines(chan_info *cp, graphics_context *ax, point_t *points, int num, int *colors, int axis_y0, color_t default_color);
+void setup_graphics_context(chan_info *cp, graphics_context *ax);
 void set_color_scale(mus_float_t val);
 void set_color_inverted(bool val);
 void set_color_cutoff(mus_float_t val);
@@ -44,18 +44,19 @@ void set_spectro_y_scale(mus_float_t val);
 void set_spectro_z_scale(mus_float_t val);
 void view_color_orientation_callback(GtkWidget * w, gpointer info);
 bool color_orientation_dialog_is_active(void);
-GtkWidget *start_color_orientation_dialog(bool managed);
+GtkWidget *make_color_orientation_dialog(bool managed);
 void reflect_spectro(void);
 void allocate_sono_rects(int size);
 void set_sono_rectangle(int j, int color, int x, int y, int width, int height);
-void draw_sono_rectangles(axis_context *ax, int color, int jmax);
-void draw_spectro_line(axis_context *ax, int color, int x0, int y0, int x1, int y1);
+void draw_sono_rectangles(graphics_context *ax, int color, int jmax);
+void draw_spectro_line(graphics_context *ax, int color, int x0, int y0, int x1, int y1);
 void allocate_color_map(int colormap);
 void check_colormap_sizes(int size);
 void initialize_colormap(void);
 void reflect_color_list(bool setup_time);
 void set_with_gl(bool val, bool with_dialogs);
 void g_init_gxdraw(void);
+gchar* scale_double_format_callback(GtkScale *w, gdouble val, gpointer data);
 
 
 
@@ -69,16 +70,14 @@ int listener_height(void);
 int listener_width(void);
 void goto_listener(void);
 int save_listener_text(FILE *fp);
-void listener_delete_text(int new_end);
 void append_listener_text(int end, const char *msg);
 void listener_append(const char *msg);
 void listener_append_and_prompt(const char *msg);
 void clear_listener(void);
 void set_listener_text_font(void);
+bool listener_colorized(void);
+bool listener_set_colorized(bool val);
 void g_init_gxlistener(void);
-GtkWidget *snd_entry_new(GtkWidget *container, snd_entry_bg_t with_white_background);
-bool highlight_unbalanced_paren(void);
-void connect_mouse_to_text(GtkWidget *text);
 
 
 
@@ -89,22 +88,24 @@ void snd_doit(int argc, char **argv);
 void auto_update_restart(void);
 void save_colors(FILE *Fp);
 
-#ifdef SND_AS_WIDGET
-  GtkWidget *snd_as_widget(int argc, char **argv, GtkWidget *parent, void (*error_func)(const char *));
-#endif
-
 
 
 /* -------- snd-gmenu.c -------- */
 
 GtkWidget *add_menu(void);
-void reflect_play_stop_in_popup_menu(void);
 void reflect_play_selection_stop(void);
 void g_init_gxmenu(void);
 GtkWidget *menu_widget(int which_menu);
 void check_menu_labels(int key, int state, bool extended);
-void popup_menu_from(GtkWidget *w, GdkEventButton *ev, gpointer data, int snd, int chn);
 GtkWidget *get_help_menu_widget(void);
+void show_toolbar(void);
+void hide_toolbar(void);
+void add_tooltip(GtkWidget *w, const char *tip);
+void post_basic_popup_menu(void *ev);
+void post_lisp_popup_menu(void *ev);
+void post_selection_popup_menu(void *ev);
+widget_t make_file_print_dialog(bool managed, bool direct_to_printer);
+
 
 
 /* -------- snd-gfft.c -------- */
@@ -114,7 +115,7 @@ void set_fft_window_alpha(mus_float_t val);
 void set_transform_size(mus_long_t val);
 void set_fft_window(mus_fft_window_t val);
 void set_wavelet_type(int val);
-GtkWidget *fire_up_transform_dialog(bool managed);
+GtkWidget *make_transform_dialog(bool managed);
 bool transform_dialog_is_active(void);
 
 void set_transform_type(int val);
@@ -134,24 +135,13 @@ void reflect_log_freq_start_in_transform_dialog(void);
 void reflect_min_db_in_transform_dialog(void);
 gboolean spin_button_focus_callback(GtkWidget *w, GdkEventCrossing *ev, gpointer unknown);
 gboolean spin_button_unfocus_callback(GtkWidget *w, GdkEventCrossing *ev, gpointer unknown);
-
-
-/* -------- snd-gdrop.c -------- */
-
-void add_drop(GtkWidget *w, 
-	      void (*watcher)(GtkWidget *w, const char *message, int x, int y, void *data), 
-	      void *context);
-void add_drag_and_drop(GtkWidget *w, 
-		       void (*drop_watcher)(GtkWidget *w, const char *message, int x, int y, void *data), 
-		       void (*drag_watcher)(GtkWidget *w, const char *message, int x, int y, drag_style_t dtype, void *data), 
-		       void *context);
-void g_init_gxdrop(void);
+void post_fft_popup_menu(void *ev);
 
 
 
 /* -------- snd-gregion.c -------- */
 
-void update_region_browser(bool grf_too);
+int update_region_browser(bool grf_too);
 void reflect_play_region_stop(int n);
 bool region_browser_is_active(void);
 void delete_region_and_update_browser(int n);
@@ -165,20 +155,18 @@ int region_dialog_region(void);
 char *regrow_get_label(void *ur);
 int regrow_get_pos(void *ur);
 void g_init_gxregion(void);
+void view_files_add_directory(widget_t dialog, const char *dirname);
+bool view_files_has_files(void);
+void view_files_callback(GtkWidget *w, gpointer info);
+void view_files_unplay(void);
 
 
 /* -------- snd-gxbitmaps.c -------- */
 
-const char **mini_bomb_bits(int n);
-const char **mini_glass_bits(int n);
-const char **mini_lock_bits(void);
-const char **blank_bits(void);
 const char **snd_icon_bits(void);
-const char **speed_l_bits(void);
-const char **speed_r_bits(void);
-const char **stop_sign_bits(void);
+enum {SND_PNG_LOCK, SND_PNG_STOP, SND_PNG_BOMB, SND_PNG_RIGHT_ARROW, SND_PNG_LEFT_ARROW, SND_PNG_BLANK, SND_PNG_SPEAKER};
+cairo_surface_t *snd_icon(int which);
 
-void make_icons_transparent(const char *basic_color);
 
 
 /* -------- snd-gxcolormaps.c -------- */
@@ -187,10 +175,14 @@ char *colormap_name(int n);
 bool is_colormap(int n);
 int num_colormaps(void);
 void get_current_color(int colormap, int n, rgb_t *r, rgb_t *g, rgb_t *b);
+#if HAVE_GL
 rgb_t *color_map_reds(int index);
 rgb_t *color_map_greens(int index);
 rgb_t *color_map_blues(int index);
+#endif
 void g_init_gxcolormaps(void);
+void phases_rgb(float x, rgb_t *r, rgb_t *g, rgb_t *b);
+
 
 
 /* -------- snd-gchn.c -------- */
@@ -211,29 +203,17 @@ void set_z_scrollbars(chan_info *cp, axis_info *ap);
 void change_gzy(mus_float_t val, chan_info *cp);
 mus_float_t gsy_value(chan_info *cp);
 mus_float_t gsy_size(chan_info *cp);
-bool fixup_cp_cgx_ax_wn(chan_info *cp);
 void reflect_edit_history_change(chan_info *cp);
 void reflect_edit_counter_change(chan_info *cp);
 gboolean graph_key_press(GtkWidget *w, GdkEventKey *event, gpointer data);
 int add_channel_window(snd_info *sound, int channel, int chan_y, int insertion, GtkWidget *main, fw_button_t arrows, bool with_events);
-void set_peak_numbers_font(chan_info *cp);
-void set_bold_peak_numbers_font(chan_info *cp);
-void set_tiny_numbers_font(chan_info *cp);
-color_t get_foreground_color(axis_context *ax);
-void set_foreground_color(axis_context *ax, color_info *color);
+void set_peak_numbers_font(chan_info *cp, graphics_context *ax);
+void set_bold_peak_numbers_font(chan_info *cp, graphics_context *ax);
+void set_tiny_numbers_font(chan_info *cp, graphics_context *ax);
+color_t get_foreground_color(graphics_context *ax);
+void set_foreground_color(graphics_context *ax, color_info *color);
 gc_t *copy_GC(chan_info *cp);
 gc_t *erase_GC(chan_info *cp);
-void free_fft_pix(chan_info *cp);
-bool restore_fft_pix(chan_info *cp, axis_context *ax);
-void save_fft_pix(chan_info *cp, axis_context *ax, int fwidth, int fheight, int x0, int y0);
-#if USE_CAIRO
-  void free_cursor_pix(chan_info *cp);
-  bool restore_cursor_pix(chan_info *cp, axis_context *ax);
-  void save_cursor_pix(chan_info *cp, axis_context *ax, int fwidth, int fheight, int x0, int y0);
-  void free_sono_cursor_pix(chan_info *cp);
-  bool restore_sono_cursor_pix(chan_info *cp, axis_context *ax);
-  void save_sono_cursor_pix(chan_info *cp, axis_context *ax, int fwidth, int fheight, int x0, int y0);
-#endif
 void cleanup_cw(chan_info *cp);
 void change_channel_style(snd_info *sp, channel_style_t new_style);
 void color_chan_components(color_t color, slider_choice_t which_component);
@@ -246,7 +226,13 @@ void g_init_gxchn(void);
 /* -------- snd-gfind.c -------- */
 
 void edit_find_callback(GtkWidget *w, gpointer info);
-void set_find_dialog_label(const char *str);
+void find_dialog(chan_info *cp);
+void find_dialog_set_label(const char *str);
+void stop_search_if_error(const char *msg, void *data);
+void errors_to_find_text(const char *msg, void *data);
+void find_dialog_stop_label(bool show_stop);
+bool find_dialog_is_active(void);
+
 void save_find_dialog_state(FILE *fd);
 void g_init_gxfind(void);
 
@@ -266,22 +252,19 @@ int number_height(PangoFontDescription *font);
 int label_height(bool use_tiny_font);
 int sg_text_width(const char *txt, PangoFontDescription *font);
 int mark_name_width(const char *txt);
-void clear_window(axis_context *ax);
+void clear_window(graphics_context *ax);
 void highlight_color(GtkWidget *w);
 void raise_dialog(GtkWidget *w);
 void set_button_label(GtkWidget *label, const char *str);
 void set_label(GtkWidget *label, const char *str);
-void sg_left_justify_button(GtkWidget *button);
-void sg_left_justify_label(GtkWidget *label);
 void check_for_event(void);
 void set_title(const char *title);
 void goto_window(GtkWidget *text);
 
 void gc_set_foreground(gc_t *gp, color_info *color);
 void gc_set_background(gc_t *gp, color_info *color);
-void gc_set_foreground_xor(gc_t *gp, color_info *col1, color_info *col2);
-void gc_set_function(gc_t *gp, GdkFunction op);
-gc_t *gc_new(GdkDrawable *wn);
+void gc_set_colors(gc_t *gp, color_info *col1, color_info *col2);
+gc_t *gc_new(void);
 
 void color_cursor(color_info *color);
 void color_marks(color_info *color);
@@ -295,15 +278,34 @@ void widget_modify_bg(GtkWidget *w, GtkStateType type, color_t color);
 void widget_modify_fg(GtkWidget *w, GtkStateType type, color_t color);
 void widget_modify_base(GtkWidget *w, GtkStateType type, color_t color);
 color_t rgb_to_color(mus_float_t r, mus_float_t g, mus_float_t b);
-#if USE_CAIRO
+
+#if (!GTK_CHECK_VERSION(3, 0, 0))
   GdkColor *rgb_to_gdk_color(color_t col);
 #endif
+void add_highlight_button_style(GtkWidget *w);
+void add_center_button_style(GtkWidget *w);
+void add_toolbar_style(GtkWidget *w);
+void add_menu_style(GtkWidget *w);
+void add_paned_style(GtkWidget *w);
+void add_red_scale_style(GtkWidget *w);
+void add_green_scale_style(GtkWidget *w);
+void add_blue_scale_style(GtkWidget *w);
+void add_white_button_style(GtkWidget *w);
+void add_listener_style(GtkWidget *w);
+void add_dialog_style(GtkWidget *w);
+void add_check_button_style(GtkWidget *w);
+void add_entry_style(GtkWidget *w);
 
 void recolor_graph(chan_info *cp, bool selected);
 void set_sensitive(GtkWidget *wid, bool val);
 void set_toggle_button(GtkWidget *wid, bool val, bool passed, void *data);
+#if GTK_CHECK_VERSION(3, 0, 0)
+int widget_height(GtkWidget *w);
+int widget_width(GtkWidget *w);
+#else
 guint16 widget_height(GtkWidget *w);
 guint16 widget_width(GtkWidget *w);
+#endif
 void set_widget_height(GtkWidget *w, guint16 height);
 void set_widget_width(GtkWidget *w, guint16 width);
 gint16 widget_x(GtkWidget *w);
@@ -318,20 +320,14 @@ void reset_user_int_data(GObject *obj, int data);
 gpointer get_user_data(GObject *obj);
 int get_user_int_data(GObject *obj);
 void set_stock_button_label(GtkWidget *w, const char *new_label);
-GtkWidget *sg_button_new_from_stock_with_label(const char *text, const gchar *stock_id);
+bool cursor_set_blinks(GtkWidget *w, bool blinks);
 
 char *sg_get_text(GtkWidget *w, int start, int end);
-void sg_set_cursor(GtkWidget *w, int position);
 void sg_text_insert(GtkWidget *w, const char *text);
-int sg_cursor_position(GtkWidget *w);
 GtkWidget *make_scrolled_text(GtkWidget *parent, bool editable, int add_choice, bool resize);
 void sg_make_resizable(GtkWidget *w);
-idle_t add_work_proc(GtkFunction func, gpointer data);
+idle_t add_work_proc(GSourceFunc func, gpointer data);
 GtkWidget *snd_gtk_dialog_new(void);
-GtkWidget *snd_gtk_entry_label_new(const char *label, color_info *color);
-GtkWidget *make_info_widget(void);
-void info_widget_display(GtkWidget *w, const char *message);
-void info_widget_set_size(GtkWidget *w, int size);
 GtkWidget *snd_gtk_highlight_label_new(const char *label);
 void widget_int_to_text(GtkWidget *w, int val);
 void widget_mus_long_t_to_text(GtkWidget *w, mus_long_t val);
@@ -346,7 +342,16 @@ void slist_clear(slist *lst);
 void slist_append(slist *lst, const char *name);
 void slist_moveto(slist *lst, int row);
 void slist_select(slist *lst, int row);
-char *slist_selection(slist *lst);
+
+#if GTK_CHECK_VERSION(3, 0, 0)
+cairo_t *make_cairo(GdkWindow *win);
+#else
+cairo_t *make_cairo(GdkDrawable *win);
+#endif
+void free_cairo(cairo_t *cr);
+void init_gtk(void);
+
+
 
 
 /* -------- snd-gsnd.c -------- */
@@ -360,13 +365,7 @@ void show_lock(snd_info *sp);
 void hide_lock(snd_info *sp);
 void start_bomb(snd_info *sp);
 void stop_bomb(snd_info *sp);
-void show_bomb(snd_info *sp);
-void hide_bomb(snd_info *sp);
-void goto_minibuffer(snd_info *sp);
-void set_minibuffer_string(snd_info *sp, const char *str, bool update);
-void set_minibuffer_cursor_position(snd_info *sp, int pos);
-char *get_minibuffer_string(snd_info *sp);
-void make_minibuffer_label(snd_info *sp, const char *str);
+void set_status(snd_info *sp, const char *str, bool update);
 void set_play_button(snd_info *sp, bool val);
 void play_button_pause(bool pausing);
 void syncb(snd_info *sp, int on);
@@ -402,8 +401,13 @@ void finish_progress_report(chan_info *cp);
 void progress_report(chan_info *cp, mus_float_t pct);
 void g_init_gxsnd(void);
 void reflect_sound_selection(snd_info *sp);
-void display_minibuffer_error(snd_info *sp, const char *str);
-void clear_minibuffer_error(snd_info *sp);
+void make_controls_dialog(void);
+void update_sound_label(snd_info *sp);
+
+GtkWidget *snd_entry_new(GtkWidget *container, GtkWidget *prev, snd_entry_bg_t with_white_background);
+GtkWidget *snd_entry_new_with_size(GtkWidget *container, int size);
+void connect_mouse_to_text(GtkWidget *text);
+
 
 
 /* -------- snd-gmix.c -------- */
@@ -413,12 +417,12 @@ GtkWidget *make_mix_dialog(void);
 void reflect_mix_play_stop(void);
 int mix_dialog_mix(void);
 void mix_dialog_set_mix(int id);
-void make_speaker_icons_transparent(const char *bg_line);
+
 
 
 /* -------- snd-genv.c -------- */
 
-axis_info *enved_make_axis(const char *name, axis_context *ax, int ex0, int ey0, int width, int height, 
+axis_info *enved_make_axis(const char *name, graphics_context *ax, int ex0, int ey0, int width, int height, 
 			   mus_float_t xmin, mus_float_t xmax, mus_float_t ymin, mus_float_t ymax, printing_t printing);
 void display_enved_env_with_selection(env *e, const char *name, int x0, int y0, int width, int height, bool dots, printing_t printing);
 void set_enved_redo_sensitive(bool val);
@@ -426,19 +430,19 @@ void set_enved_revert_sensitive(bool val);
 void set_enved_undo_sensitive(bool val);
 void set_enved_save_sensitive(bool val);
 void set_enved_show_sensitive(bool val);
+void enved_reflect_selection(bool on);
 void make_scrolled_env_list(void);
 void enved_reflect_peak_env_completion(snd_info *sp);
 void new_active_channel_alert(void);
 void env_redisplay(void);
 void env_redisplay_with_print(void);
 void update_enved_background_waveform(chan_info *cp);
-void enved_print(char *name);
 GtkWidget *create_envelope_editor(void);
-void set_enved_clip_p(bool val);
+void set_enved_clipping(bool val);
 void reflect_enved_style(void);
 void set_enved_base(mus_float_t val);
 void set_enved_target(enved_target_t val);
-void set_enved_wave_p(bool val);
+void set_enved_with_wave(bool val);
 void set_enved_in_dB(bool val);
 bool enved_dialog_is_active(void);
 void set_enved_filter_order(int order);
@@ -447,18 +451,17 @@ void g_init_gxenv(void);
 
 
 
-/* -------- snd-grec.c -------- */
-
-widget_t record_file(void);
-
-
-
 /* -------- snd-gfile.c -------- */
 
-char *get_file_dialog_sound_attributes(file_data *fdat, int *srate, int *chans, int *type, int *format, mus_long_t *location, mus_long_t *samples, int min_chan);
-void alert_new_file(void);
+void cleanup_file_monitor(void);
+void *unmonitor_file(void *watcher);
+void monitor_sound(snd_info *sp);
+
+char *get_file_dialog_sound_attributes(file_data *fdat, int *srate, int *chans, mus_header_t *header_type, 
+				       mus_sample_t *sample_type, mus_long_t *location, mus_long_t *samples, int min_chan);
 widget_t make_open_file_dialog(read_only_t read_only, bool managed);
 widget_t make_sound_save_as_dialog(bool managed);
+void make_channel_extract_dialog(int chan);
 widget_t make_selection_save_as_dialog(bool managed);
 widget_t make_region_save_as_dialog(bool managed);
 widget_t make_new_file_dialog(bool managed);
@@ -466,30 +469,32 @@ widget_t make_mix_file_dialog(bool managed);
 widget_t make_insert_file_dialog(bool managed);
 GtkWidget *edit_header(snd_info *sp);
 void save_edit_header_dialog_state(FILE *fd);
-void cleanup_edit_header_watcher(void);
-void cleanup_new_file_watcher(void);
 void set_open_file_play_button(bool val);
 void g_init_gxfile(void);
-void clear_deleted_snd_info(struct dialog_play_info *fd);
+void clear_deleted_snd_info(void *fd);
 void reflect_just_sounds(void);
 void save_file_dialog_state(FILE *fd);
 widget_t post_it(const char *subject, const char *str);
+void post_it_append(const char *str);
 void save_post_it_dialog_state(FILE *fd);
 void reflect_region_in_save_as_dialog(void);
-void mouse_enter_label(void *r, int type);
-void mouse_leave_label(void *r, int type);
-
-
-/* -------- snd-gprint.c -------- */
-
-void file_print_callback(GtkWidget *w, gpointer info);
-widget_t make_file_print_dialog(bool managed, bool direct_to_printer);
-void save_print_dialog_state(FILE *fd);
+void reflect_selection_in_save_as_dialog(bool on);
+void save_edits_now(snd_info *sp);
+void unpost_unsaved_edits_if_any(snd_info *sp);
+void unpost_file_has_changed_if_any(snd_info *sp);
+void changed_file_dialog(snd_info *sp);
+void reflect_save_as_src(bool val);
+void reflect_save_as_auto_comment(bool val);
+void reflect_save_as_sound_selection(const char *sound_name);
+void add_drag_and_drop(GtkWidget *w, 
+		       void (*drop_watcher)(GtkWidget *w, const char *message, int x, int y, void *data), 
+		       void (*drag_watcher)(GtkWidget *w, const char *message, int x, int y, drag_style_t dtype, void *data), 
+		       void *context);
 
 
 /* -------- snd-gprefs.c -------- */
 
-widget_t start_preferences_dialog(void);
+widget_t make_preferences_dialog(void);
 
 #endif
 
