@@ -413,13 +413,10 @@
     (let ((labelled-snds ()))
       
       (define (kmg num)
-	(if (<= num 0)
-	    "disk full!"
-	    (if (> num 1024)
-		(if (> num (* 1024 1024))
-		    (format #f "space: ~6,3FG" (/ num (* 1024.0 1024.0)))
-		    (format #f "space: ~6,3FM" (/ num 1024.0)))
-		(format #f "space: ~10DK" num))))
+	(cond ((<= num 0)      "disk full!")
+	      ((<= num 1024)   (format #f "space: ~10DK" num))
+	      ((> num 1048576) (format #f "space: ~6,3FG" (/ num (* 1024.0 1024.0))))
+	      (else            (format #f "space: ~6,3FM" (/ num 1024.0)))))
       
       (define (show-label data)
 	(if (sound? (car data))
@@ -428,10 +425,10 @@
 	      (g_timeout_add 10000 show-label data) ; every 10 seconds recheck space
 	      0)))
       
-      (define (find-if pred l)
-	(cond ((null? l) #f)
-	      ((pred (car l)) (car l))
-	      (else (find-if pred (cdr l)))))
+      (define (find-if pred lst)
+	(cond ((null? lst) #f)
+	      ((pred (car lst)) (car lst))
+	      (else (find-if pred (cdr lst)))))
       
       (lambda (hook)
 	;; (show-disk-space snd) adds a label to snd's status-area area showing the current free space (for use with after-open-hook)
@@ -480,9 +477,9 @@
   (define snd-clock-icon
     (lambda (snd hour)
       (let* ((window (GDK_WINDOW (gtk_widget_get_window ((sound-widgets snd) 8))))
-	     (cr (gdk_cairo_create window))
-	     (bg (color->list *basic-color*)))
-	(cairo_set_source_rgb cr (car bg) (cadr bg) (caddr bg))
+	     (cr (gdk_cairo_create window)))
+	(let ((bg (color->list *basic-color*)))
+	  (cairo_set_source_rgb cr (car bg) (cadr bg) (caddr bg)))
 	(cairo_rectangle cr 0 0 16 16) ; icon bg
 	(cairo_fill cr)
 	(cairo_set_source_rgb cr 1.0 1.0 1.0)
@@ -503,11 +500,10 @@
   (define (snd-happy-face snd progress)
     (let* ((window (GDK_WINDOW (gtk_widget_get_window ((sound-widgets snd) 8))))
 	   (cr (gdk_cairo_create window))
-	   (bg (color->list *basic-color*))
 	   (fc (list 1.0 progress 0.0)))
-      
-      ;; overall background
-      (cairo_set_source_rgb cr (car bg) (cadr bg) (caddr bg))
+      (let ((bg (color->list *basic-color*)))
+        ;; overall background
+        (cairo_set_source_rgb cr (car bg) (cadr bg) (caddr bg)))
       (cairo_rectangle cr 0 0 16 16)
       (cairo_fill cr)
       
@@ -565,15 +561,18 @@
       
       (define (find-free-dialog ds)
 	(and (pair? ds)
-	     (if (not (cadr (car ds)))
+	     (pair? (car ds))
+	     (pair? (cdar ds))
+	     (if (cadar ds)
+		 (find-free-dialog (cdr ds))
 		 (begin
 		   (set! ((car ds) 1) #t)
-		   (caar ds))
-		 (find-free-dialog (cdr ds)))))
+		   (caar ds)))))
+
       (lambda args
 	;; (file-select func title dir filter help)
-	(let* ((func (and (> (length args) 0) (args 0)))
-	       (title (if (> (length args) 1) (args 1) "select file"))
+	(let* ((func (and (pair? args) (args 0)))
+	       (title (if (and (pair? args) (pair? (cdr args))) (args 1) "select file"))
 	       (dir (if (> (length args) 2) (args 2) "."))
 	       (dialog (or (find-free-dialog file-selector-dialogs)
 			   (GTK_FILE_CHOOSER_DIALOG (gtk_file_chooser_dialog_new
@@ -803,16 +802,15 @@
 	(case type
 	  ((text)
 	   ;; add a horizontal pair: label text
-	   (let ((label (gtk_label_new var-label))
-		 (hbox (gtk_box_new GTK_ORIENTATION_HORIZONTAL 0))
-		 (text (gtk_label_new "")))
-	     (gtk_box_pack_start (GTK_BOX pane) hbox #f #f 2)
-	     (gtk_widget_show hbox)
-	     (gtk_box_pack_start (GTK_BOX hbox) label #f #f 6)
-	     
-	     (gtk_widget_set_halign (GTK_WIDGET label) GTK_ALIGN_START)
-	     (gtk_widget_show label)
-	     (gtk_box_pack_start (GTK_BOX hbox) text #t #t 6)
+	   (let ((text (gtk_label_new "")))
+	     (let ((hbox (gtk_box_new GTK_ORIENTATION_HORIZONTAL 0)))
+	       (let ((label (gtk_label_new var-label)))
+		 (gtk_box_pack_start (GTK_BOX pane) hbox #f #f 2)
+		 (gtk_widget_show hbox)
+		 (gtk_box_pack_start (GTK_BOX hbox) label #f #f 6)
+		 (gtk_widget_set_halign (GTK_WIDGET label) GTK_ALIGN_START)
+		 (gtk_widget_show label))
+	       (gtk_box_pack_start (GTK_BOX hbox) text #t #t 6))
 	     (gtk_widget_set_halign (GTK_WIDGET text) GTK_ALIGN_START)
 	     (gtk_widget_show text)
 	     text))
@@ -866,9 +864,7 @@
 		  (set! (data loc) var)
 		  (if (time-graph? snd) (update-time-graph snd))
 		  (if (transform-graph? snd) (update-transform-graph snd))
-		  (if (= (+ loc 1) len)
-		      (set! (cursor snd 0) 0)
-		      (set! (cursor snd 0) (+ loc 1))))
+		  (set! (cursor snd 0) (if (= (+ loc 1) len) 0 (+ loc 1))))
 		(if (GTK_IS_PROGRESS_BAR (car widget))
 		    ;; "thermometer"
 		    (let ((y0 (cadr widget))
