@@ -1601,7 +1601,6 @@ static Xen g_bind_key_1(Xen key, Xen state, Xen code, Xen cx_extended, Xen origi
     set_keymap_entry(k, s, 0, Xen_undefined, e, NULL, NULL);
   else 
     {
-      char buf[256];
       int args;
       const char *comment = NULL, *prefs = NULL;
       args = Xen_required_args(code);
@@ -1617,7 +1616,11 @@ static Xen g_bind_key_1(Xen key, Xen state, Xen code, Xen cx_extended, Xen origi
 	}
       if (Xen_is_string(origin))
 	comment = Xen_string_to_C_string(origin); 
-      else comment = make_key_name(buf, 256, k, s, e);
+      else
+	{
+	  char buf[256];
+	  comment = make_key_name(buf, 256, k, s, e);
+	}
       if (Xen_is_string(prefs_info)) prefs = Xen_string_to_C_string(prefs_info);
       set_keymap_entry(k, s, args, code, e, comment, prefs);
     }
