@@ -53,11 +53,11 @@
 
 #include "snd-strings.h"
 
-#define SND_DATE "30-Nov-15"
+#define SND_DATE "9-May-16"
 #ifndef SND_VERSION
-#define SND_VERSION "16.1"
+#define SND_VERSION "16.5"
 #endif
 #define SND_MAJOR_VERSION "16"
-#define SND_MINOR_VERSION "1"
+#define SND_MINOR_VERSION "5"
 
 #endif
