@@ -155,11 +155,10 @@
 	  (procedure? (car lst)))
       lst
       (begin
-	(if (symbol? (car lst))
-	    (set-car! lst (symbol->value (car lst)))
-	    (if (pair? (car lst))
-		(set-car! lst (apply lambda '(x) `((or (,(caar lst) x) (,(cadar lst) x)))))
-		(set-car! lst #f)))
+	(set-car! lst (if (symbol? (car lst))
+			  (symbol->value (car lst))
+			  (and (pair? (car lst))
+			       (apply lambda '(x) `((or (,(caar lst) x) (,(cadar lst) x)))))))
 	(map-values (cdr lst)))))
 
 (define baddies '(exit emergency-exit abort autotest 
@@ -169,7 +168,7 @@
 		  throw symbol-table load
 		  global-environment current-environment make-procedure-with-setter procedure-with-setter? make-rectangular
 		  
-		  copy fill! hash-table-set! vector-set! let-set! hash-table-size
+		  copy fill! hash-table-set! vector-set! let-set!
 		  
 		  mock-number mock-pair mock-string mock-char mock-vector 
 		  mock-symbol mock-port mock-hash-table m 
@@ -195,10 +194,10 @@
 		      ;(format *stderr* ";~A...~%" sym)
 		      (set! low bottom)
 		      (if (positive? (cdr argn))
-			  (let ((sig (if (eq? sym 'append)
-					 (let ((lst (list 'list?)))
-					   (set-cdr! lst lst))
-					 (copy (procedure-signature f)))))
+			  (let ((sig (cond ((eq? sym 'append)
+					    (let ((lst (list 'list?)))
+					      (set-cdr! lst lst)))
+					   ((procedure-signature f) => copy))))
 			    (map-values sig)
 			    (autotest f () 0 top (if (pair? sig) (cdr sig) ()))))))))))))
 
