@@ -60,8 +60,8 @@
 (call-valgrind)
 
 (when (file-exists? "test.table")
-  (system "mv test.table old-test.table")
-  (load "compare-calls.scm")
-  (combine-latest))
+  (system "mv test.table old-test.table"))
+(load "compare-calls.scm")
+(combine-latest)
 
 (exit)
\ No newline at end of file
